/** \file acquisitionthread.cpp
 *  \brief This file contains the implementation of the AcquisitionThread class.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "acquisitionthread.h"
#include "enumreference.h"

#include <QMessageBox>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>


AcquisitionThread::AcquisitionThread( QObject *pParent, Dc1394Camera *pCamera )
 : QThread( pParent )
 , m_pCamera( pCamera )
 , m_bSisReadout(false)
 , m_FramesFinishedSemaphore( 0 )
 , m_FramesReadySemaphore( 0 )
{
    Q_CHECK_PTR( pCamera );
    Q_CHECK_PTR( pCamera->psCamInternal() );

    if ( pipe( m_pAbortPipeFileDesc ) == -1 )
	{
        throw( QString( "could not create pipe for abort signals" ) );
	}
    if ( fcntl( m_pAbortPipeFileDesc[0], F_SETFL, O_NONBLOCK ) != 0 )
	{
        throw( QString ( "fcntl on abort pipe failed" ) );
	}

    m_eAcquisitionMode = CC1394_ACQ_FREERUN;

    // read oneshot settings
    dc1394bool_t eBoolVal;
    dc1394error_t eRet;
    eRet = dc1394_video_get_one_shot( m_pCamera->psCamInternal(), &eBoolVal );
    LOGFCALL_LIBDC( "dc1394_video_get_one_shot", eRet );
    if( DC1394_TRUE == eBoolVal )
    {
        m_eAcquisitionMode = CC1394_ACQ_SINGLE_SHOT;
    }

    // read multishot settings
    uint32_t nFrameCount;
    eRet = dc1394_video_get_multi_shot( m_pCamera->psCamInternal(), &eBoolVal, &nFrameCount );
    LOGFCALL_LIBDC( "dc1394_video_get_multi_shot", eRet );
    m_nMultishotFrameCount = (int) nFrameCount;
    if( DC1394_TRUE == eBoolVal )
    {
        m_eAcquisitionMode = CC1394_ACQ_MULTI_SHOT;
    }

    m_bAvtCamera = ( DC1394_AVT_VENDOR_ID == m_pCamera->psCamInternal()->vendor_id );
    if( m_bAvtCamera )
    {
        // read camera (model) id for AVT cameras
        uint32_t nDummy1,nDummy2,nDummy3;
        eRet = dc1394_avt_get_version( m_pCamera->psCamInternal(), &nDummy1, &nDummy2, &m_nCameraId, &nDummy3 );
        LOGFCALL_LIBDC( "dc1394_avt_get_version", eRet );
        if( DC1394_SUCCESS != eRet )
        {
            m_nCameraId = 0;
        }

        // read SIS capabilities
        eRet = dc1394_avt_get_sis_data_inquiry( m_pCamera->smartFeatureInquiry(), &m_sSisInquiry );
        LOGFCALL_LIBDC( "dc1394_avt_get_sis_data_inquiry", eRet );
        if( DC1394_SUCCESS != eRet )
        {
            memset( &m_sSisInquiry, 0, sizeof(dc1394_avt_sis_data));
        }
    }
    else
    {
        memset( &m_sSisInquiry, 0, sizeof(dc1394_avt_sis_data));
    }
}


AcquisitionThread::~AcquisitionThread()
{
	if ( isRunning() )
	{
		abort();
	}

    close( m_pAbortPipeFileDesc[0] );
    close( m_pAbortPipeFileDesc[1] );
}


void AcquisitionThread::setMultishotFrameCount( int nCount ) 
{
    if(nCount>0) 
    {
        m_nMultishotFrameCount = nCount; 
        dc1394error_t eRet = dc1394_video_set_multi_shot( m_pCamera->psCamInternal(),
                                                          (uint32_t) m_nMultishotFrameCount,
                                                          DC1394_OFF );
        LOGFCALL_LIBDC( "dc1394_video_set_multi_shot", eRet );
    } 
}


void AcquisitionThread::abort( bool bWait )
{
	if ( !isRunning() )
	{
		return;
	}

	/* stop camera's iso stream */
	dc1394error_t eRet;
    eRet = dc1394_video_set_transmission ( m_pCamera->psCamInternal(), DC1394_OFF );
    LOGFCALL_LIBDC ( "dc1394_video_set_transmission", ( dc1394error_t ) eRet );

	/* signal abortion to acquisition thread */
	char nDummy = 0xFF;
    write( m_pAbortPipeFileDesc[1], &nDummy, 1 );
	if ( bWait )
	{
		wait();
	}
}


/*
 * thread entry point - the actual acquisition loop
 */
void AcquisitionThread::run()
{
	dc1394error_t eRet;

	emit acquisitionStateChanged(true);
    /* Reset semaphore that counts the frames delivered to the display unit. */
    m_FramesFinishedSemaphore.acquire ( m_FramesFinishedSemaphore.available() );
    m_FramesReadySemaphore.acquire ( m_FramesReadySemaphore.available() );

	/* make sure capture is disabled */
    eRet = dc1394_video_set_transmission( m_pCamera->psCamInternal(), DC1394_OFF );
    LOGFCALL_LIBDC( "dc1394_video_set_transmission", ( dc1394error_t ) eRet );
    usleep( 10000 );

    /* setup acquisition mode: singleshot/multishot/freerun */
    int nExpectedFrames = -1;

    eRet = dc1394_video_set_multi_shot( m_pCamera->psCamInternal(),
                                        m_nMultishotFrameCount, 
                                        CC1394_ACQ_MULTI_SHOT == m_eAcquisitionMode ? DC1394_ON : DC1394_OFF );
    LOGFCALL_LIBDC( "dc1394_video_set_multi_shot", ( dc1394error_t ) eRet );

    eRet = dc1394_video_set_one_shot( m_pCamera->psCamInternal(),
                                      CC1394_ACQ_SINGLE_SHOT == m_eAcquisitionMode ? DC1394_ON : DC1394_OFF );
    LOGFCALL_LIBDC( "dc1394_video_set_one_shot", ( dc1394error_t ) eRet );

    if( CC1394_ACQ_SINGLE_SHOT == m_eAcquisitionMode )
    {
        nExpectedFrames = 1;
    }
    if( CC1394_ACQ_MULTI_SHOT == m_eAcquisitionMode )
    {
        nExpectedFrames = m_nMultishotFrameCount;
    } 

    /* Setup Capture - We currently allocate the channel and not the iso bandwidth. Program crashes may leave a channel occupied. */
    eRet = dc1394_capture_setup( m_pCamera->psCamInternal(), NUM_BUFFERS, DC1394_CAPTURE_FLAGS_CHANNEL_ALLOC );
    LOGFCALL_LIBDC( "dc1394_capture_setup", ( dc1394error_t ) eRet );

    int nCamFileDesc = dc1394_capture_get_fileno( m_pCamera->psCamInternal() );

    eRet = dc1394_video_set_transmission( m_pCamera->psCamInternal(), DC1394_ON );
    LOGFCALL_LIBDC( "dc1394_video_set_transmission", ( dc1394error_t ) eRet );

	dc1394video_frame_t *pFrame;

	/* reset abort pipe */
	char pDummyData[16];
	for ( ;; )
	{
        if ( read( m_pAbortPipeFileDesc[0], pDummyData, 16 ) <= 0 )
		{
			break;
		}
	}

	/* data structures for select() */
	fd_set read_fd_set;

	unsigned int nFramesAvailable = NUM_BUFFERS; //!< number of frame buffers currently available for dequeueing
	int retval;

	/* enter main loop */
	for(;;)
	{
        FD_ZERO( &read_fd_set );
        FD_SET( m_pAbortPipeFileDesc[0], &read_fd_set );
        FD_SET( nCamFileDesc, &read_fd_set );

        /* wait for new image and/or abort events, no timeout */
        retval = select( std::max( m_pAbortPipeFileDesc[0],nCamFileDesc ) + 1,
                         &read_fd_set, NULL, NULL, NULL);

		if ( retval == -1 )
		{
            throw( QString( tr( "select() failed!" ) ) );
		}
		if ( retval == 0 )
		{
            throw( QString( tr( "timeout!" ) ) );
		}
        if ( FD_ISSET( m_pAbortPipeFileDesc[0], &read_fd_set ) )
		{
			/* termination requested */
			break;
		}

        if ( FD_ISSET( nCamFileDesc, &read_fd_set ) )
		{
			/* frame is available - request it */
			nFramesAvailable--;
            eRet = dc1394_capture_dequeue( m_pCamera->psCamInternal(), DC1394_CAPTURE_POLICY_POLL, &pFrame );
			if ( eRet != DC1394_SUCCESS )
			{
                LOGFCALL_LIBDC( "dc1394_capture_dequeue", eRet );
			}
			if ( pFrame == NULL )
			{
				/* no frame returned - this should never happen as we made sure that frames are available */
				throw( QString ( tr ( "Error!" ) ) );
			}

            if( m_bAvtCamera )
            {
                eRet = dc1394_avt_adjust_frames( m_nCameraId, pFrame );
                if ( eRet != DC1394_SUCCESS )
                {
                    LOGFCALL_LIBDC( "dc1394_avt_adjust_frames", eRet );
                }
            }

            if( m_bSisReadout && m_bAvtCamera )
            {
                dc1394_avt_sis_data sSIS;
                dc1394_avt_get_sis_data( m_pCamera->smartFeatureInquiry(), pFrame, (int16_t) m_nSisLine, &sSIS );
                QString strSisText = Sis2Text( &m_sSisInquiry, &sSIS );
                Logger::log( strSisText );
            }

			/* queue frame buffer to be returned to libdc later */
			m_queueFramesInUse.enqueue( pFrame );
			if ( 0 < receivers( SIGNAL( newImageReceived( dc1394video_frame_t*,
								      QSemaphore*, 
                                      QSemaphore* ) ) ) )
			{
                /* send frame to display unit ( which will raise m_FinishedFramesSemaphore when finished ) */
                emit newImageReceived( pFrame, &m_FramesFinishedSemaphore, &m_FramesReadySemaphore );
                m_FramesReadySemaphore.release();
			}
			else
			{
				/* no receiver connected, frame can be returned immediately */
                m_FramesFinishedSemaphore.release();
			}

			/* are there queued frame buffers that can be returned to libdc? */
            unsigned int n = m_FramesFinishedSemaphore.available();
			if ( n>0 )
			{
				/* return n frames to the libdc*/
				for ( unsigned int i=0; i<n; i++ )
				{
                    eRet = dc1394_capture_enqueue( m_pCamera->psCamInternal(), m_queueFramesInUse.dequeue() );
					if ( eRet != DC1394_SUCCESS )
					{
                        LOGFCALL_LIBDC( "dc1394_capture_enqueue", eRet );
					}
				}
                m_FramesFinishedSemaphore.acquire( n );
				nFramesAvailable = nFramesAvailable + n;
			}

            /* keep track of the number of frames when single/multi-shot acquisition is performed */
            if( -1 != nExpectedFrames )
            {
                nExpectedFrames--;
                if( 0 == nExpectedFrames )
                {
                    break;
                }
            }

			/* are there frame buffers available for libdc? */
			if ( nFramesAvailable == 0 )
			{
				/* no, wait (blocking) until one frame is released */
                m_FramesFinishedSemaphore.acquire();
				/* return one frame to the libdc */
                eRet = dc1394_capture_enqueue( m_pCamera->psCamInternal(), m_queueFramesInUse.dequeue() );
				if ( eRet != DC1394_SUCCESS )
				{
                    LOGFCALL_LIBDC( "dc1394_capture_enqueue", ( dc1394error_t ) eRet );
				}
				nFramesAvailable++;
			}
		}
	}

	/* wait until all delivered frames are processed and return them to libdc */
	unsigned int nFramesQueued = NUM_BUFFERS - nFramesAvailable;
    m_FramesFinishedSemaphore.acquire( nFramesQueued );
	for ( unsigned int i=0; i<nFramesQueued; i++ )
	{
        eRet = dc1394_capture_enqueue( m_pCamera->psCamInternal(), m_queueFramesInUse.dequeue() );
		if ( eRet != DC1394_SUCCESS )
		{
            LOGFCALL_LIBDC( "dc1394_capture_enqueue", eRet );
		}
	}

	/* shut down capture */
    eRet = dc1394_capture_stop( m_pCamera->psCamInternal() );
    LOGFCALL_LIBDC( "dc1394_capture_stop", eRet );
    eRet = dc1394_video_set_transmission( m_pCamera->psCamInternal(), DC1394_OFF );
    LOGFCALL_LIBDC( "dc1394_video_set_transmission", eRet );
	emit acquisitionStateChanged(false);
}


/*
 * helper function building a string description from image SIS data
 */
QString AcquisitionThread::Sis2Text( dc1394_avt_sis_data *psSisInq, dc1394_avt_sis_data *psSIS )
{
    QString str;
    str = "SIS data found:\n";

    if( psSisInq->CycleTime.m_all != 0 )
    {
        str += " Timestamp: " + QString::number(psSIS->CycleTime.m.Seconds) + ":" + QString::number(psSIS->CycleTime.m.Cycles) +
               ":" + QString::number(psSIS->CycleTime.m.Offset) + " ";
    }
    if( psSisInq->FrameCounter != 0 )
    {
        str += " Framecounter: " + QString::number(psSIS->FrameCounter) + " ";
    }
    if( psSisInq->TriggerCounter != 0 )
    {
        str += " Triggercounter: " + QString::number(psSIS->TriggerCounter) + " ";
    }
    if( psSisInq->AOIHeight != 0 || psSisInq->AOIWidth != 0 || psSisInq->AOITop != 0 || psSisInq->AOILeft != 0 )
    {
        str += " AOI (L:T:W:H): ";
        str += QString::number(psSIS->AOILeft) + ":" + QString::number(psSIS->AOITop) + ":" +
               QString::number(psSIS->AOIWidth) + ":" + QString::number(psSIS->AOIHeight) + " ";
    }
    if( psSisInq->Shutter != 0 )
    {
        str += " Shutter: " + QString::number(psSIS->Shutter) + " ";
    }
    if( psSisInq->Gain != 0 )
    {
        str += " Gain: " + QString::number(psSIS->Gain) + " ";
    }
    if( psSisInq->OutputState[0] != 0 || psSisInq->OutputState[1] != 0 || psSisInq->OutputState[2] != 0 || psSisInq->OutputState[3] != 0 )
    {
        str += QString(" Outputs: ") + ( (psSIS->OutputState[0]==0) ? "-":"X" ) +
                                       ( (psSIS->OutputState[1]==0) ? "-":"X" ) +
                                       ( (psSIS->OutputState[2]==0) ? "-":"X" ) +
                                       ( (psSIS->OutputState[3]==0) ? "-":"X" ) + " ";
    }
    if( psSisInq->InputState[0] != 0 || psSisInq->InputState[1] != 0 )
    {
        str += QString(" Inputs: ") + ( (psSIS->InputState[0]==0) ? "-":"X" ) +
                                      ( (psSIS->InputState[1]==0) ? "-":"X" ) + " ";
    }
    if( psSisInq->SequenceIndex != 0 )
    {
        str += " Seq.Ind.: " + QString::number(psSIS->SequenceIndex) + " ";
    }
    if( psSisInq->ColorCoding != 0 )
    {
        str += " ColCoding: " + EnumReference::LookupColorCodingName( (dc1394color_coding_t) psSIS->ColorCoding ) + " ";
    }
    if( psSisInq->SerialNumber != 0 )
    {
        str += " SN: " + QString::number(psSIS->SerialNumber) + " ";
    }
    if( psSisInq->UserValue != 0 )
    {
        str += " UserVal: " + QString::number(psSIS->UserValue) + " ";
    }

    return str;
}


