/** \file acquisitionthread.h
 *  \brief This file contains the definition of the AcquisitionThread class.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef ACQUISITIONTHREAD_H
#define ACQUISITIONTHREAD_H

#include <QWidget>
#include <QThread>
#include <QSemaphore>
#include <QQueue>

#include "dc1394camera.h"

#define NUM_BUFFERS 5


typedef enum
{
    CC1394_ACQ_FREERUN,
    CC1394_ACQ_SINGLE_SHOT,
    CC1394_ACQ_MULTI_SHOT
} cc1394_acquisition_mode;

/*
 * \brief thread running the libdc dequeue/enqueue loop
 *
 * This thread acquires images from libdc in a loop calling dc1394_capture_dequeue/dc1394_capture_enqueue.
 * Image processing is decoupled from this loop without additional copy operations on frame buffers.
 * The original buffer received from libdc is emitted as a Qt signal along with a semaphore. Once the receiver
 * finished accessing the frame buffer, release() has to be called on this semaphore.
 *
 * Released frame buffers are not returned to libdc, while the thread is waiting for a new image. This may
 * result in a slight performance loss. One additional frame buffer should compensate this.
 *
 * Images must be released in the same order as they have been emitted.
 *
 * Acquisition may be aborted at any time.
 */
class AcquisitionThread : public QThread
{
	Q_OBJECT
public:
    AcquisitionThread( QObject *pParent, Dc1394Camera *pCamera );
	virtual ~AcquisitionThread();

    void setMultishotFrameCount( int nCount );
    int  multishotFrameCount() { return m_nMultishotFrameCount; }

    void setAcquisitionMode( cc1394_acquisition_mode eMode ) { m_eAcquisitionMode = eMode; }
    cc1394_acquisition_mode acquisitionMode() { return m_eAcquisitionMode; }

    void setSisReadout( bool bEnabled, int nLine ) { m_bSisReadout = bEnabled; m_nSisLine = nLine; }
    bool getSisReadout() { return m_bSisReadout; }

	void abort( bool bWait = false );
	void run();

signals:
    void newImageReceived( dc1394video_frame_t* pFrame, QSemaphore *pFramesFinishedSemaphore, QSemaphore* pFramesReadySemaphore );
	void acquisitionStateChanged( bool bAcquisitionActive );

private:
    QString Sis2Text( dc1394_avt_sis_data *psSisInq, dc1394_avt_sis_data *psSIS );
    Dc1394Camera        *m_pCamera;
    bool                m_bAvtCamera;
    uint32_t            m_nCameraId; // Camera (Model) Id for AVT Cameras
    dc1394_avt_sis_data m_sSisInquiry;

    int                             m_nMultishotFrameCount;
    cc1394_acquisition_mode         m_eAcquisitionMode;

    bool                            m_bSisReadout;
    int                             m_nSisLine;

	QMutex	m_ThreadControlMutex;

	/** 
     * Semaphore which represents the number of frames that can be returned to the libdc.
     * Will be raised when the receiver object finishes frame processing.
     * Will be lowered by the acquisition thread when frames are actually returned to the libdc.
     */
    QSemaphore m_FramesFinishedSemaphore;

    /**
    * Semaphore indicating the number newFrameReceived() signals that are queued for the
    * receiver object. For a viewer application, only the latest frame is of interest, so
    * frames should be dropped when this value is greater as 1.
    *
    * Will be raised by the acq thread when a new frame is sent. Will be lowered by the receiver
    * when the frame is started to be processed.
    *
    * The aqcuisition thread does not rely on this semaphore, it's just there to inform the receiver.
    */
    QSemaphore m_FramesReadySemaphore;

	/**
	* FIFO queue of frames which have been delivered to the receiver.
	*/
	QQueue<dc1394video_frame_t*> m_queueFramesInUse; 

	/**
	* file descriptors of the 'abort pipe'. This pipe is used as an abort event for the image
	* acquisition thread: If a single byte is written to it, the acquisition will be aborted.
	*/
	int m_pAbortPipeFileDesc[2]; 	


};

#endif
