/** \file autofncaoicontrol.cpp
 *  \brief This file contains the implementation of the feature control class AutofncAoiControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "autofncaoicontrol.h"
#include "areaselection.h"



AutofncAoiControl::AutofncAoiControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_AutofncAoiControl()
{
    setupUi ( this );
    connect( m_pAreaFrame, SIGNAL( areaChanged() ), this, SLOT( onAreaChange() ) );
    connect( m_pLeftSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onLeftChange(int) ) );
    connect( m_pTopSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onTopChange(int) ) );
    connect( m_pWidthSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onWidthChange(int) ) );
    connect( m_pHeightSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onHeightChange(int) ) );

    connect( m_pEnableCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onEnableClicked(bool) ) );
    connect( m_pShowAreaCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onShowAreaClicked(bool) ) );
}


AutofncAoiControl::~AutofncAoiControl()
{
}


void AutofncAoiControl::setCamera( Dc1394Camera*      pCamera,
                                   AcquisitionThread* /*pThread*/ )
{
    dc1394error_t eRet;
    uint32_t nLeft, nTop;
    uint32_t nWidth, nHeight;
    uint32_t nXUnit, nYUnit;
    dc1394switch_t eEnable, eShowImg;

    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    /* check feature support */
    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->AutoFunctionAOI )
    {
        /* Auto Function AOI is not supported */
        setEnabled(false);
        return;
    }

    /* get AF AOI units */
    eRet = dc1394_avt_get_autofunc_aoi_info( m_pCamObject->psCamInternal(), &nXUnit, &nYUnit );
    LOGFCALL_LIBDC ( "dc1394_avt_get_autofunc_aoi_info", ( dc1394error_t ) eRet );

    m_nXUnit = (int) nXUnit;
    m_nYUnit = (int) nYUnit;

    /* get current image size */
    eRet = dc1394_get_image_size_from_video_mode( pCamera->psCamInternal(), pCamera->getVideoMode(), &nWidth, &nHeight);
    LOGFCALL_LIBDC ( "dc1394_get_image_size_from_video_mode", ( dc1394error_t ) eRet );

    m_nMaxX =  ((((int)nWidth-1)/m_nXUnit)+1)*m_nXUnit;
    m_nMaxY =  ((((int)nHeight-1)/m_nYUnit)+1)*m_nYUnit;

    /* get current autofnc AOI setting */
    eRet = dc1394_avt_get_autofunc_aoi( m_pCamObject->psCamInternal(), &eEnable, &eShowImg,  &nLeft, &nTop, &nWidth, &nHeight );
    LOGFCALL_LIBDC ( "dc1394_avt_get_autofunc_aoi", eRet );
    m_nWidth = (int) nWidth;
    m_nHeight = (int) nHeight;
    m_nLeft = (int) nLeft;
    m_nTop = (int) nTop;
    m_bEnable = (DC1394_ON==eEnable);
    m_bShowArea = (DC1394_ON==eShowImg);

    m_pEnableCheckBox->setChecked( m_bEnable );
    m_pShowAreaCheckBox->setChecked( m_bShowArea );
    m_pShowAreaCheckBox->setEnabled( m_bEnable );

    /* prevent spinboxes from sending change signals */
    m_pLeftSpinBox->blockSignals ( true );
    m_pTopSpinBox->blockSignals ( true );
    m_pWidthSpinBox->blockSignals ( true );
    m_pHeightSpinBox->blockSignals ( true );

    /* set spinbox range */
    m_pLeftSpinBox->setRange ( 0, m_nMaxX );
    m_pTopSpinBox->setRange ( 0, m_nMaxY );
    m_pWidthSpinBox->setRange ( m_nXUnit, m_nMaxX );
    m_pHeightSpinBox->setRange ( m_nYUnit, m_nMaxY );

    /* adjust spinbox step sizes */
    m_pLeftSpinBox->setSingleStep ( ( int ) nXUnit );
    m_pTopSpinBox->setSingleStep ( ( int ) nYUnit );
    m_pWidthSpinBox->setSingleStep ( ( int ) nXUnit );
    m_pHeightSpinBox->setSingleStep ( ( int ) nYUnit );

    /* adjust spinbox contents */
    m_pLeftSpinBox->setValue ( ( int ) m_nLeft );
    m_pTopSpinBox->setValue ( ( int ) m_nTop );
    m_pWidthSpinBox->setValue ( ( int ) m_nWidth );
    m_pHeightSpinBox->setValue ( ( int ) m_nHeight );

    /* re-enable signals */
    m_pLeftSpinBox->blockSignals ( false );
    m_pTopSpinBox->blockSignals ( false );
    m_pWidthSpinBox->blockSignals ( false );
    m_pHeightSpinBox->blockSignals ( false );

    /* init area selection control */
    m_pAreaFrame->setAreaLeft(m_nLeft);
    m_pAreaFrame->setAreaTop(m_nTop);
    m_pAreaFrame->setAreaWidth( m_nWidth );
    m_pAreaFrame->setAreaHeight( m_nHeight );
    m_pAreaFrame->setMaxAreaWidth( m_nMaxX );
    m_pAreaFrame->setMaxAreaHeight( m_nMaxY );
    m_pAreaFrame->setAreaSizeFixed( false );
    m_pAreaFrame->init();

    adjustSpinboxRanges();

    setEnabled(true);
}

void AutofncAoiControl::onVideoModeSwitched_autoconnect()
{
    dc1394error_t eRet;
    uint32_t nLeft, nTop;
    uint32_t nWidth, nHeight;
    dc1394switch_t eEnable, eShowImg;

    if ( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    /* check feature support */
    if ( DC1394_FALSE == m_pCamObject->smartFeatureInquiry()->AutoFunctionAOI )
    {
        /* Auto Function AOI is not supported */
        setEnabled(false);
        return;
    }

    /* get current image size */
    eRet = dc1394_get_image_size_from_video_mode( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nWidth, &nHeight);
    LOGFCALL_LIBDC ( "dc1394_get_image_size_from_video_mode", ( dc1394error_t ) eRet );

    m_nMaxX =  ((((int)nWidth-1)/m_nXUnit)+1)*m_nXUnit;
    m_nMaxY =  ((((int)nHeight-1)/m_nYUnit)+1)*m_nYUnit;

    /* get current autofnc AOI setting */
    eRet = dc1394_avt_get_autofunc_aoi( m_pCamObject->psCamInternal(), &eEnable, &eShowImg,  &nLeft, &nTop, &nWidth, &nHeight );
    LOGFCALL_LIBDC ( "dc1394_avt_get_autofunc_aoi", eRet );
    m_nWidth = (int) nWidth;
    m_nHeight = (int) nHeight;
    m_nLeft = (int) nLeft;
    m_nTop = (int) nTop;
    m_bEnable = (DC1394_ON==eEnable);
    m_bShowArea = (DC1394_ON==eShowImg);

    m_pEnableCheckBox->setChecked( m_bEnable );
    m_pShowAreaCheckBox->setChecked( m_bShowArea );
    m_pShowAreaCheckBox->setEnabled( m_bEnable );

    /* prevent spinboxes from sending change signals */
    m_pLeftSpinBox->blockSignals ( true );
    m_pTopSpinBox->blockSignals ( true );
    m_pWidthSpinBox->blockSignals ( true );
    m_pHeightSpinBox->blockSignals ( true );

    /* set spinbox range */
    m_pLeftSpinBox->setRange ( 0, m_nMaxX );
    m_pTopSpinBox->setRange ( 0, m_nMaxY );
    m_pWidthSpinBox->setRange ( m_nXUnit, m_nMaxX );
    m_pHeightSpinBox->setRange ( m_nYUnit, m_nMaxY );

    /* adjust spinbox contents */
    m_pLeftSpinBox->setValue ( ( int ) m_nLeft );
    m_pTopSpinBox->setValue ( ( int ) m_nTop );
    m_pWidthSpinBox->setValue ( ( int ) m_nWidth );
    m_pHeightSpinBox->setValue ( ( int ) m_nHeight );

    adjustSpinboxRanges();

    /* re-enable signals */
    m_pLeftSpinBox->blockSignals ( false );
    m_pTopSpinBox->blockSignals ( false );
    m_pWidthSpinBox->blockSignals ( false );
    m_pHeightSpinBox->blockSignals ( false );

    /* init area selection control */
    m_pAreaFrame->setAreaLeft(m_nLeft);
    m_pAreaFrame->setAreaTop(m_nTop);
    m_pAreaFrame->setAreaWidth( m_nWidth );
    m_pAreaFrame->setAreaHeight( m_nHeight );
    m_pAreaFrame->setMaxAreaWidth( m_nMaxX );
    m_pAreaFrame->setMaxAreaHeight( m_nMaxY );
    m_pAreaFrame->init();

    setEnabled(true);
}

void AutofncAoiControl::onImageSizeChanged_autoconnect( int nImgWidth, int nImgHeight )
{
    dc1394error_t eRet;
    uint32_t nLeft, nTop;
    uint32_t nWidth, nHeight;
    dc1394switch_t eEnable, eShowImg;

    m_nMaxX =  ((((int)nImgWidth-1)/m_nXUnit)+1)*m_nXUnit;
    m_nMaxY =  ((((int)nImgHeight-1)/m_nYUnit)+1)*m_nYUnit;

    /* get current autofnc AOI setting */
    eRet = dc1394_avt_get_autofunc_aoi( m_pCamObject->psCamInternal(), &eEnable, &eShowImg,  &nLeft, &nTop, &nWidth, &nHeight );
    LOGFCALL_LIBDC ( "dc1394_avt_get_autofunc_aoi", eRet );
    m_nWidth = (int) nWidth;
    m_nHeight = (int) nHeight;
    m_nLeft = (int) nLeft;
    m_nTop = (int) nTop;
    m_bEnable = (DC1394_ON==eEnable);
    m_bShowArea = (DC1394_ON==eShowImg);

    m_pEnableCheckBox->setChecked( m_bEnable );
    m_pShowAreaCheckBox->setChecked( m_bShowArea );
    m_pShowAreaCheckBox->setEnabled( m_bEnable );

    /* prevent spinboxes from sending change signals */
    m_pLeftSpinBox->blockSignals ( true );
    m_pTopSpinBox->blockSignals ( true );
    m_pWidthSpinBox->blockSignals ( true );
    m_pHeightSpinBox->blockSignals ( true );

    /* set spinbox range */
    m_pLeftSpinBox->setRange ( 0, m_nMaxX );
    m_pTopSpinBox->setRange ( 0, m_nMaxY );
    m_pWidthSpinBox->setRange ( m_nXUnit, m_nMaxX );
    m_pHeightSpinBox->setRange ( m_nYUnit, m_nMaxY );

    /* adjust spinbox contents */
    m_pLeftSpinBox->setValue ( ( int ) m_nLeft );
    m_pTopSpinBox->setValue ( ( int ) m_nTop );
    m_pWidthSpinBox->setValue ( ( int ) m_nWidth );
    m_pHeightSpinBox->setValue ( ( int ) m_nHeight );

    adjustSpinboxRanges();

    /* re-enable signals */
    m_pLeftSpinBox->blockSignals ( false );
    m_pTopSpinBox->blockSignals ( false );
    m_pWidthSpinBox->blockSignals ( false );
    m_pHeightSpinBox->blockSignals ( false );

    /* init area selection control */
    m_pAreaFrame->setAreaLeft(m_nLeft);
    m_pAreaFrame->setAreaTop(m_nTop);
    m_pAreaFrame->setAreaWidth( m_nWidth );
    m_pAreaFrame->setAreaHeight( m_nHeight );
    m_pAreaFrame->setMaxAreaWidth( m_nMaxX );
    m_pAreaFrame->setMaxAreaHeight( m_nMaxY );
    m_pAreaFrame->init();
}

void AutofncAoiControl::onStateChangedByTwinControl()
{
    AutofncAoiControl* pSender = qobject_cast<AutofncAoiControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nLeft = pSender->m_nLeft;
    m_nTop = pSender->m_nTop;
    m_nWidth = pSender->m_nWidth;
    m_nHeight = pSender->m_nHeight;

    m_pLeftSpinBox->setValue( m_nLeft );
    m_pTopSpinBox->setValue( m_nTop );
    m_pWidthSpinBox->setValue( m_nWidth );
    m_pHeightSpinBox->setValue( m_nHeight );

    adjustSpinboxRanges();

    m_pAreaFrame->setAreaLeft( m_nLeft );
    m_pAreaFrame->setAreaTop( m_nTop );
    m_pAreaFrame->setAreaWidth( m_nWidth );
    m_pAreaFrame->setAreaHeight( m_nHeight );

    m_pShowAreaCheckBox->setEnabled( m_bEnable );
}

void AutofncAoiControl::adjustSpinboxRanges()
{
    m_pLeftSpinBox->setMaximum( m_nMaxX - m_nWidth );
    m_nLeft = m_pLeftSpinBox->value();
    m_pWidthSpinBox->setMaximum( m_nMaxX - m_nLeft );
    m_nWidth = m_pWidthSpinBox->value();
    m_pTopSpinBox->setMaximum( m_nMaxY - m_nHeight );
    m_nTop = m_pTopSpinBox->value();
    m_pHeightSpinBox->setMaximum( m_nMaxY - m_nTop );
    m_nHeight = m_pHeightSpinBox->value();
}


void AutofncAoiControl::pollStateFromCamera()
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    dc1394error_t eRet;
    uint32_t nLeft;
    uint32_t nTop;
    uint32_t nWidth;
    uint32_t nHeight;
    dc1394switch_t eEnable, eShowImg;

    /* get current AF AOI setting */
    eRet = dc1394_avt_get_autofunc_aoi( m_pCamObject->psCamInternal(), &eEnable, &eShowImg,  &nLeft, &nTop, &nWidth, &nHeight );
    LOGFCALL_LIBDC ( "dc1394_avt_get_autofunc_aoi", eRet );
    m_nWidth = (int) nWidth;
    m_nHeight = (int) nHeight;
    m_nLeft = (int) nLeft;
    m_nTop = (int) nTop;
    m_bEnable = (DC1394_ON==eEnable);
    m_bShowArea = (DC1394_ON==eShowImg);

    m_pEnableCheckBox->setChecked( m_bEnable );
    m_pShowAreaCheckBox->setChecked( m_bShowArea );
    m_pShowAreaCheckBox->setEnabled( m_bEnable );

    /* prevent spinboxes from sending change signals */
    m_pLeftSpinBox->blockSignals ( true );
    m_pTopSpinBox->blockSignals ( true );
    m_pWidthSpinBox->blockSignals ( true );
    m_pHeightSpinBox->blockSignals ( true );

    /* adjust spinbox contents */
    m_pLeftSpinBox->setValue ( ( int ) m_nLeft );
    m_pTopSpinBox->setValue ( ( int ) m_nTop );
    m_pWidthSpinBox->setValue ( ( int ) m_nWidth );
    m_pHeightSpinBox->setValue ( ( int ) m_nHeight );

    adjustSpinboxRanges();

    /* re-enable signals */
    m_pLeftSpinBox->blockSignals ( false );
    m_pTopSpinBox->blockSignals ( false );
    m_pWidthSpinBox->blockSignals ( false );
    m_pHeightSpinBox->blockSignals ( false );

    /* adjust area selection control */
    m_pAreaFrame->setAreaLeft(m_nLeft);
    m_pAreaFrame->setAreaTop(m_nTop);
    m_pAreaFrame->setAreaWidth( m_nWidth );
    m_pAreaFrame->setAreaHeight( m_nHeight );

    m_pShowAreaCheckBox->setEnabled( m_bEnable );
}


void AutofncAoiControl::onLeftChange( int nValue )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nLeft == nValue )
    {
        return;
    }

    m_nLeft = nValue;

    adjustSpinboxRanges();

    m_pAreaFrame->setAreaLeft( m_nLeft );

    /* configure camera */
    dc1394error_t eRet  = dc1394_avt_set_autofunc_aoi( m_pCamObject->psCamInternal(),
                                                       m_bEnable? DC1394_ON : DC1394_OFF,
                                                       m_bShowArea? DC1394_ON : DC1394_OFF,
                                                       (uint32_t) m_nLeft,
                                                       (uint32_t) m_nTop,
                                                       (uint32_t) m_nWidth,
                                                       (uint32_t) m_nHeight );
    LOGFCALL_LIBDC ( "dc1394_avt_set_autofunc_aoi", eRet );
    
    emit featureStateChanged();
    emit AutofncAoiChanged();
}


void AutofncAoiControl::onTopChange( int nValue )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nTop == nValue )
    {
        return;
    }

    m_nTop = nValue;

    adjustSpinboxRanges();

    m_pAreaFrame->setAreaTop( m_nTop );

    /* configure camera */
    dc1394error_t eRet  = dc1394_avt_set_autofunc_aoi( m_pCamObject->psCamInternal(),
                                                       m_bEnable? DC1394_ON : DC1394_OFF,
                                                       m_bShowArea? DC1394_ON : DC1394_OFF,
                                                       (uint32_t) m_nLeft,
                                                       (uint32_t) m_nTop,
                                                       (uint32_t) m_nWidth,
                                                       (uint32_t) m_nHeight );
    LOGFCALL_LIBDC ( "dc1394_avt_set_autofunc_aoi", eRet );

    emit featureStateChanged();
    emit AutofncAoiChanged();
}


void AutofncAoiControl::onWidthChange( int nValue )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nWidth == nValue )
    {
        return;
    }
    m_nWidth = nValue;

    adjustSpinboxRanges();

    m_pAreaFrame->setAreaWidth( m_nWidth );

    /* configure camera */
    dc1394error_t eRet  = dc1394_avt_set_autofunc_aoi( m_pCamObject->psCamInternal(),
                                                       m_bEnable? DC1394_ON : DC1394_OFF,
                                                       m_bShowArea? DC1394_ON : DC1394_OFF,
                                                       (uint32_t) m_nLeft,
                                                       (uint32_t) m_nTop,
                                                       (uint32_t) m_nWidth,
                                                       (uint32_t) m_nHeight );
    LOGFCALL_LIBDC ( "dc1394_avt_set_autofunc_aoi", eRet );

    emit featureStateChanged();
    emit AutofncAoiChanged();
}


void AutofncAoiControl::onHeightChange( int nValue )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nHeight == nValue )
    {
        return;
    }
    m_nHeight = nValue;

    adjustSpinboxRanges();

    m_pAreaFrame->setAreaHeight( m_nHeight );

   /* configure camera */
    dc1394error_t eRet  = dc1394_avt_set_autofunc_aoi( m_pCamObject->psCamInternal(),
                                                       m_bEnable? DC1394_ON : DC1394_OFF,
                                                       m_bShowArea? DC1394_ON : DC1394_OFF,
                                                       (uint32_t) m_nLeft,
                                                       (uint32_t) m_nTop,
                                                       (uint32_t) m_nWidth,
                                                       (uint32_t) m_nHeight );
    LOGFCALL_LIBDC ( "dc1394_avt_set_autofunc_aoi", eRet );

    emit featureStateChanged();
    emit AutofncAoiChanged();
}


void AutofncAoiControl::onAreaChange  ()
{
    m_nTop = m_pAreaFrame->areaTop();
    m_nTop -= m_nTop % m_nYUnit;

    m_nLeft = m_pAreaFrame->areaLeft();
    m_nLeft -= m_nLeft % m_nXUnit;

    m_nWidth = m_pAreaFrame->areaWidth();
    m_nWidth -= m_nWidth % m_nXUnit;
    m_nWidth = qMax( m_nWidth, m_nXUnit );

    m_nHeight = m_pAreaFrame->areaHeight();
    m_nHeight -= m_nHeight % m_nYUnit;
    m_nHeight = qMax( m_nHeight, m_nYUnit );
    
    m_pLeftSpinBox->blockSignals ( true );
    m_pTopSpinBox->blockSignals ( true );
    m_pTopSpinBox->setValue( m_nTop );
    m_pLeftSpinBox->setValue( m_nLeft );
    m_pLeftSpinBox->blockSignals ( false );
    m_pTopSpinBox->blockSignals ( false );

    m_pWidthSpinBox->blockSignals ( true );
    m_pHeightSpinBox->blockSignals ( true );
    m_pWidthSpinBox->setValue( m_nWidth );
    m_pHeightSpinBox->setValue( m_nHeight );
    m_pWidthSpinBox->blockSignals ( false );
    m_pHeightSpinBox->blockSignals ( false );

    adjustSpinboxRanges();

    /* configure camera */
    dc1394error_t eRet  = dc1394_avt_set_autofunc_aoi( m_pCamObject->psCamInternal(),
                                                       m_bEnable? DC1394_ON : DC1394_OFF,
                                                       m_bShowArea? DC1394_ON : DC1394_OFF,
                                                       (uint32_t) m_nLeft,
                                                       (uint32_t) m_nTop,
                                                       (uint32_t) m_nWidth,
                                                       (uint32_t) m_nHeight );
    LOGFCALL_LIBDC ( "dc1394_avt_set_autofunc_aoi", eRet );

    emit featureStateChanged();
    emit AutofncAoiChanged();
}


void AutofncAoiControl::onEnableClicked( bool bChecked )
{
    if(bChecked!=m_bEnable)
    {
        m_bEnable = bChecked;

        /* configure camera */
        dc1394error_t eRet  = dc1394_avt_set_autofunc_aoi( m_pCamObject->psCamInternal(),
                                                       m_bEnable? DC1394_ON : DC1394_OFF,
                                                       m_bShowArea? DC1394_ON : DC1394_OFF,
                                                       (uint32_t) m_nLeft,
                                                       (uint32_t) m_nTop,
                                                       (uint32_t) m_nWidth,
                                                       (uint32_t) m_nHeight );
        LOGFCALL_LIBDC ( "dc1394_avt_set_autofunc_aoi", eRet );

        emit featureStateChanged();
        emit AutofncAoiChanged();

        m_pShowAreaCheckBox->setEnabled( m_bEnable );
    }
}

void AutofncAoiControl::onShowAreaClicked( bool bChecked )
{
    if(bChecked!=m_bShowArea)
    {
        m_bShowArea = bChecked;

        /* configure camera */
        dc1394error_t eRet  = dc1394_avt_set_autofunc_aoi( m_pCamObject->psCamInternal(),
                                                       m_bEnable? DC1394_ON : DC1394_OFF,
                                                       m_bShowArea? DC1394_ON : DC1394_OFF,
                                                       (uint32_t) m_nLeft,
                                                       (uint32_t) m_nTop,
                                                       (uint32_t) m_nWidth,
                                                       (uint32_t) m_nHeight );
        LOGFCALL_LIBDC ( "dc1394_avt_set_autofunc_aoi", eRet );

        emit featureStateChanged();
        emit AutofncAoiChanged();
    }
}


