/** \file autogaincontrol.cpp
 *  \brief This file contains the implementation of the feature control class AutoGainControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "autogaincontrol.h"

AutoGainControl::AutoGainControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_AutoGainControl()
{
    setupUi(this);

    connect( m_pMinSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onChangeMin(int) ) );
    connect( m_pMaxSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onChangeMax(int) ) );
}


AutoGainControl::~AutoGainControl()
{
}


void AutoGainControl::setCamera( Dc1394Camera*      pCamera, 
                                 AcquisitionThread* /*pThread*/ )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    /* check feature support */
    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->AutoGain )
    {
        /* auto shutter control is not supported */
        setEnabled(false);
        return;
    }

    /* read valid range from gain feature info */
    const dc1394feature_info_t* psGainInfo = pCamera->getFeatureInfo( DC1394_FEATURE_GAIN );
    if ( DC1394_FALSE == psGainInfo->available )
    {
        /* gain is not supported */
        setEnabled(false);
        return;
    }

    m_pMinSpinBox->setMinimum( psGainInfo->min );
    m_pMaxSpinBox->setMaximum( psGainInfo->max );

    setEnabled(true);
    pollStateFromCamera();
}


void AutoGainControl::pollStateFromCamera()
{
    dc1394error_t eRet;
    uint32_t nMin, nMax;
    eRet = dc1394_avt_get_auto_gain( m_pCamObject->psCamInternal(), &nMin, &nMax );
    LOGFCALL_LIBDC ( "dc1394_avt_get_auto_shutter", eRet );

    m_nCurrentAutoGainMin = (int) nMin;
    m_nCurrentAutoGainMax = (int) nMax;

    m_pMinSpinBox->blockSignals(true);
    m_pMaxSpinBox->blockSignals(true);
    m_pMinSpinBox->setMaximum( m_nCurrentAutoGainMax - 1 );
    m_pMaxSpinBox->setMinimum( m_nCurrentAutoGainMin + 1 );
    m_pMinSpinBox->blockSignals(false);
    m_pMaxSpinBox->blockSignals(false);

    m_pMinSpinBox->setValue( m_nCurrentAutoGainMin );
    m_pMaxSpinBox->setValue( m_nCurrentAutoGainMax );
}


void AutoGainControl::onStateChangedByTwinControl(void)
{
    AutoGainControl* pSender = qobject_cast<AutoGainControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nCurrentAutoGainMin = pSender->m_nCurrentAutoGainMin;
    m_nCurrentAutoGainMax = pSender->m_nCurrentAutoGainMax;

    m_pMinSpinBox->setMaximum( m_nCurrentAutoGainMax - 1 );
    m_pMaxSpinBox->setMinimum( m_nCurrentAutoGainMin + 1 );

    m_pMinSpinBox->setValue( m_nCurrentAutoGainMin );
    m_pMaxSpinBox->setValue( m_nCurrentAutoGainMax );
}


void AutoGainControl::onChangeMin( int nVal )
{
    if( m_nCurrentAutoGainMin != nVal )
    {
        m_nCurrentAutoGainMin = nVal;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_auto_gain( m_pCamObject->psCamInternal(),
                                         (uint32_t) m_nCurrentAutoGainMin,
                                         (uint32_t) m_nCurrentAutoGainMax );
        LOGFCALL_LIBDC ( "dc1394_avt_set_auto_gain", eRet );

        m_pMaxSpinBox->setMinimum( m_nCurrentAutoGainMin + 1 );
    }
}


void AutoGainControl::onChangeMax( int nVal )
{
    if( m_nCurrentAutoGainMax != nVal )
    {
        m_nCurrentAutoGainMax = nVal;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_auto_gain( m_pCamObject->psCamInternal(),
                                         (uint32_t) m_nCurrentAutoGainMin,
                                         (uint32_t) m_nCurrentAutoGainMax );
        LOGFCALL_LIBDC ( "dc1394_avt_set_auto_gain", eRet );

        m_pMinSpinBox->setMaximum( m_nCurrentAutoGainMax - 1 );
    }
}
