/** \file dsnucontrol.cpp
 *  \brief This file contains the implementation of the feature control class DsnuControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "dsnucontrol.h"

DsnuControl::DsnuControl ( QWidget* pParent )
        : AbstractFeatureControl ( pParent )
        , Ui_DsnuControl()
{
    setupUi ( this );

    m_pSpinBox->setMinimum( 1 );
    m_pSpinBox->setMaximum( 99999 );

    connect( m_pSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onFrameCountChange(int) ) );
    connect( m_pEnableCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onEnableClicked(bool) ) );
    connect( m_pShowCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onShowImageClicked(bool) ) );
    connect( m_pPushButton, SIGNAL( clicked(bool) ), this, SLOT( onComputeClicked() ) );
}


DsnuControl::~DsnuControl()
{
}


void DsnuControl::setCamera ( Dc1394Camera*      pCamera,
                              AcquisitionThread* /*pThread*/ )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        m_bRefreshRequired = false;
        return;
    }

    /* check feature support */
    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->DSNU )
    {
        /* DSNU Correction is not supported */
        setEnabled(false);
        return;
    }

    m_bRefreshRequired = false;
    setEnabled(true);
    pollStateFromCamera();
}


void DsnuControl::pollStateFromCamera()
{ 
    dc1394error_t   eRet;
    uint32_t        nFrameCount;
    dc1394switch_t  eActive;
    dc1394bool_t    eBuildErr; /* unused */
    dc1394switch_t  eShowImage;

    eRet = dc1394_avt_get_dsnu_correction( m_pCamObject->psCamInternal(),
                                           &eActive,
                                           &eBuildErr,
                                           &nFrameCount,
                                           &eShowImage );
    LOGFCALL_LIBDC ( "dc1394_avt_get_dsnu_correction", eRet );

    m_nFrameCount = (int) nFrameCount;
    m_bActive  = ( DC1394_ON == eActive );
    m_bShowImage = ( DC1394_ON == eShowImage );

    m_pEnableCheckBox->setChecked( m_bActive );
    m_pShowCheckBox->setChecked( m_bShowImage );
    m_pSpinBox->setValue( m_nFrameCount );

    enableControls();
}


void DsnuControl::enableControls()
{
    m_pShowCheckBox->setEnabled( m_bActive && !m_bRefreshRequired );
}


void DsnuControl::onStateChangedByTwinControl ( void )
{
    DsnuControl* pSender = qobject_cast<DsnuControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nFrameCount = pSender->m_nFrameCount;
    m_bActive = pSender->m_bActive;
    m_bShowImage = pSender->m_bShowImage;

    m_pEnableCheckBox->setChecked( m_bActive );
    m_pShowCheckBox->setChecked( m_bShowImage );
    m_pSpinBox->setValue( m_nFrameCount );

    enableControls();
}


void DsnuControl::onFrameCountChange ( int nCount )
{
    if( m_nFrameCount != nCount )
    {
        m_nFrameCount = nCount;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_dsnu_correction( m_pCamObject->psCamInternal(),
                                               m_bActive? DC1394_ON : DC1394_OFF,
                                               DC1394_OFF,
                                               (uint32_t) m_nFrameCount,
                                               m_bShowImage? DC1394_ON : DC1394_OFF,
                                               DC1394_OFF,
                                               DC1394_OFF );
        LOGFCALL_LIBDC ( "dc1394_avt_set_dsnu_correction", eRet );

        emit featureStateChanged();
        emit DsnuChange( m_bActive, m_bShowImage );
    }
}


void DsnuControl::onEnableClicked ( bool bChecked )
{
    if( m_bActive != bChecked )
    {
        m_bActive = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_dsnu_correction( m_pCamObject->psCamInternal(),
                                               m_bActive? DC1394_ON : DC1394_OFF,
                                               DC1394_OFF,
                                               (uint32_t) m_nFrameCount,
                                               m_bShowImage? DC1394_ON : DC1394_OFF,
                                               DC1394_OFF,
                                               DC1394_OFF );
        LOGFCALL_LIBDC ( "dc1394_avt_set_dsnu_correction", eRet )

        enableControls();

        emit featureStateChanged();
        emit DsnuChange( m_bActive, m_bShowImage );
    }
}


void DsnuControl::onShowImageClicked ( bool bChecked )
{
    if( m_bShowImage != bChecked )
    {
        m_bShowImage = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_dsnu_correction( m_pCamObject->psCamInternal(),
                                               m_bActive? DC1394_ON : DC1394_OFF,
                                               DC1394_OFF,
                                               (uint32_t) m_nFrameCount,
                                               m_bShowImage? DC1394_ON : DC1394_OFF,
                                               DC1394_OFF,
                                               DC1394_OFF );
        LOGFCALL_LIBDC ( "dc1394_avt_set_dsnu_correction", eRet )

        emit featureStateChanged();
        emit DsnuChange( m_bActive, m_bShowImage );
    }
}

void DsnuControl::onComputeClicked()
{
    if( !m_bRefreshRequired )
    {
        dc1394error_t eRet;
        setEnabled(false);
        eRet = dc1394_avt_set_dsnu_correction( m_pCamObject->psCamInternal(),
                                               m_bActive? DC1394_ON : DC1394_OFF,
                                               DC1394_ON,
                                               (uint32_t) m_nFrameCount,
                                               m_bShowImage? DC1394_ON : DC1394_OFF,
                                               DC1394_OFF,
                                               DC1394_OFF );
        LOGFCALL_LIBDC ( "dc1394_avt_set_dsnu_correction", eRet )
        setEnabled(true);
    }
}
