/** \file hdrcontrol.cpp
 *  \brief This file contains the implementation of the feature control class HdrControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "hdrcontrol.h"

HdrControl::HdrControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_HdrControl()
{
    setupUi ( this );

    // connect controls
    connect( m_pEnableCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onEnableClicked(bool) ) );
    connect( m_pVal1SpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onKneepoint1Change(int) ) );
    connect( m_pVal2SpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onKneepoint2Change(int) ) );
    connect( m_pVal3SpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onKneepoint3Change(int) ) );
    connect( m_pCountSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onKneepointCountChange(int) ) );

    // set ranges
    m_pCountSpinBox->blockSignals( true );
    m_pCountSpinBox->setMinimum( 1 );
    m_pCountSpinBox->setMaximum( 3 );
    m_pCountSpinBox->blockSignals( false );

    m_pVal1SpinBox->blockSignals( true );
    m_pVal1SpinBox->setMinimum( 0 );
    m_pVal1SpinBox->setMaximum( ( 1<<16 ) - 1 );
    m_pVal1SpinBox->blockSignals( false );
    m_pVal1Slider->blockSignals( true );
    m_pVal1Slider->setMinimum( 0 );
    m_pVal1Slider->setMaximum( ( 1<<16 ) - 1 );
    m_pVal1Slider->blockSignals( false );

    m_pVal2SpinBox->blockSignals( true );
    m_pVal2SpinBox->setMinimum( 0 );
    m_pVal2SpinBox->setMaximum( ( 1<<16 ) - 1 );
    m_pVal2SpinBox->blockSignals( false );
    m_pVal2Slider->blockSignals( true );
    m_pVal2Slider->setMinimum( 0 );
    m_pVal2Slider->setMaximum( ( 1<<16 ) - 1 );
    m_pVal2Slider->blockSignals( false );

    m_pVal3SpinBox->blockSignals( true );
    m_pVal3SpinBox->setMinimum( 0 );
    m_pVal3SpinBox->setMaximum( ( 1<<16 ) - 1 );
    m_pVal3SpinBox->blockSignals( false );
    m_pVal3Slider->blockSignals( true );
    m_pVal3Slider->setMinimum( 0 );
    m_pVal3Slider->setMaximum( ( 1<<16 ) - 1 );
    m_pVal3Slider->blockSignals( false );
}


HdrControl::~HdrControl()
{
}


void HdrControl::setCamera( Dc1394Camera*      pCamera, 
                            AcquisitionThread* /*pThread*/ )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    /* check feature support */
    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->HDR_Mode )
    {
        /* HDR mode is not supported */
        setEnabled(false);
        return;
    }

    pollStateFromCamera();

    setEnabled(true);
}


void HdrControl::pollStateFromCamera()
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    bool bStateChanged = false;

    /* get current state */
    dc1394error_t eRet;
    dc1394bool_t eActive;
    uint32_t nCount;
    uint32_t nVal1;
    uint32_t nVal2;
    uint32_t nVal3;

    eRet = dc1394_avt_get_multiple_slope( m_pCamObject->psCamInternal(),
                                          &eActive,
                                          &nCount,
                                          &nVal1,
                                          &nVal2,
                                          &nVal3 );
    LOGFCALL_LIBDC ( "dc1394_avt_get_multiple_slope", eRet );

    if( m_nKneepoint1Val != (int) nVal1 )
    {
        m_nKneepoint1Val = nVal1;
        m_pVal1SpinBox->setValue( m_nKneepoint1Val );
        bStateChanged = true;
    }

    if( m_nKneepoint2Val != (int) nVal2 )
    {
        m_nKneepoint2Val = nVal2;
        m_pVal2SpinBox->setValue( m_nKneepoint2Val );
        bStateChanged = true;
    }

    if( m_nKneepoint3Val != (int) nVal3 )
    {
        m_nKneepoint3Val = nVal3;
        m_pVal3SpinBox->setValue( m_nKneepoint3Val );
        bStateChanged = true;
    }

    if( m_nKneepointCount != (int) nCount )
    {
        m_nKneepointCount = nCount;
        m_pCountSpinBox->setValue( m_nKneepointCount );
        bStateChanged = true;
    } 

    if( m_bActive != ( eActive == DC1394_TRUE ) )
    {
        m_bActive = ( eActive == DC1394_TRUE );
        m_pEnableCheckBox->setChecked( m_bActive );
        bStateChanged = true;
    }

    m_pVal2SpinBox->setEnabled( m_nKneepointCount > 1 );
    m_pVal3SpinBox->setEnabled( m_nKneepointCount > 2 );

    if( bStateChanged )
    {
        adjustRanges();
    }
}


void HdrControl::onStateChangedByTwinControl(void)
{
    HdrControl* pSender = qobject_cast<HdrControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nKneepointCount = pSender->m_nKneepointCount;
    m_nKneepoint1Val = pSender->m_nKneepoint1Val;
    m_nKneepoint2Val = pSender->m_nKneepoint2Val;
    m_nKneepoint3Val = pSender->m_nKneepoint3Val;
    m_bActive = pSender->m_bActive;

    m_pVal1SpinBox->setValue( m_nKneepoint1Val );
    m_pVal2SpinBox->setValue( m_nKneepoint2Val );
    m_pVal3SpinBox->setValue( m_nKneepoint3Val );
    m_pCountSpinBox->setValue( m_nKneepointCount );
    m_pEnableCheckBox->setChecked( m_bActive );

    m_pVal2SpinBox->setEnabled( m_nKneepointCount > 1 );
    m_pVal3SpinBox->setEnabled( m_nKneepointCount > 2 );

    adjustRanges();
}


void HdrControl::onKneepointCountChange( int nCount )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nKneepointCount != nCount )
    {
        m_nKneepointCount = nCount;

        /* configure camera */
        dc1394error_t eRet;
        eRet = dc1394_avt_set_multiple_slope( m_pCamObject->psCamInternal(),
                                              m_bActive? DC1394_TRUE : DC1394_FALSE,
                                              (uint32_t) m_nKneepointCount,
                                              (uint32_t) m_nKneepoint1Val,
                                              (uint32_t) m_nKneepoint2Val,
                                              (uint32_t) m_nKneepoint3Val );
        LOGFCALL_LIBDC ( "dc1394_avt_set_multiple_slope", eRet );

        m_pVal2SpinBox->setEnabled( m_nKneepointCount > 1 );
        m_pVal3SpinBox->setEnabled( m_nKneepointCount > 2 );

        adjustRanges();

        emit featureStateChanged();
    }
}


void HdrControl::onKneepoint1Change( int nValue )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nKneepoint1Val != nValue )
    {
        m_nKneepoint1Val = nValue;

        /* configure camera */
        dc1394error_t eRet;
        eRet = dc1394_avt_set_multiple_slope( m_pCamObject->psCamInternal(),
                                              m_bActive? DC1394_TRUE : DC1394_FALSE,
                                              (uint32_t) m_nKneepointCount,
                                              (uint32_t) m_nKneepoint1Val,
                                              (uint32_t) m_nKneepoint2Val,
                                              (uint32_t) m_nKneepoint3Val );
        LOGFCALL_LIBDC ( "dc1394_avt_set_multiple_slope", eRet );

        adjustRanges();

        emit featureStateChanged();
    }
}


void HdrControl::onKneepoint2Change( int nValue )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nKneepoint2Val != nValue )
    {
        m_nKneepoint2Val = nValue;

        /* configure camera */
        dc1394error_t eRet;
        eRet = dc1394_avt_set_multiple_slope( m_pCamObject->psCamInternal(),
                                              m_bActive? DC1394_TRUE : DC1394_FALSE,
                                              (uint32_t) m_nKneepointCount,
                                              (uint32_t) m_nKneepoint1Val,
                                              (uint32_t) m_nKneepoint2Val,
                                              (uint32_t) m_nKneepoint3Val );
        LOGFCALL_LIBDC ( "dc1394_avt_set_multiple_slope", eRet );

        adjustRanges();

        emit featureStateChanged();
    }
}


void HdrControl::onKneepoint3Change( int nValue )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nKneepoint3Val != nValue )
    {
        m_nKneepoint3Val = nValue;

        /* configure camera */
        dc1394error_t eRet;
        eRet = dc1394_avt_set_multiple_slope( m_pCamObject->psCamInternal(),
                                              m_bActive? DC1394_TRUE : DC1394_FALSE,
                                              (uint32_t) m_nKneepointCount,
                                              (uint32_t) m_nKneepoint1Val,
                                              (uint32_t) m_nKneepoint2Val,
                                              (uint32_t) m_nKneepoint3Val );
        LOGFCALL_LIBDC ( "dc1394_avt_set_multiple_slope", eRet );

        adjustRanges();

        emit featureStateChanged();
    }
}


void HdrControl::onEnableClicked( bool bChecked )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_bActive != bChecked )
    {
        m_bActive = bChecked;

        /* configure camera */
        dc1394error_t eRet;
        eRet = dc1394_avt_set_multiple_slope( m_pCamObject->psCamInternal(),
                                              m_bActive? DC1394_TRUE : DC1394_FALSE,
                                              (uint32_t) m_nKneepointCount,
                                              (uint32_t) m_nKneepoint1Val,
                                              (uint32_t) m_nKneepoint2Val,
                                              (uint32_t) m_nKneepoint3Val );
        LOGFCALL_LIBDC ( "dc1394_avt_set_multiple_slope", eRet );

        emit featureStateChanged();
    }
}


void HdrControl::adjustRanges()
{
    int nVal1Max = m_nKneepointCount>1 ? m_nKneepoint2Val-1 : ( 1<<16 ) - 1;
    if( m_pVal1SpinBox->maximum() != nVal1Max )
    {
        m_pVal1SpinBox->blockSignals( true );
        m_pVal1SpinBox->setMaximum( nVal1Max );
        m_pVal1SpinBox->blockSignals( false );
        m_pVal1Slider->blockSignals( true );
        m_pVal1Slider->setMaximum( nVal1Max );
        m_pVal1Slider->blockSignals( false );
    }

    if ( m_nKneepointCount < 2 )
    {
        return;
    }

    if( m_pVal2SpinBox->minimum() != m_nKneepoint1Val+1 )
    {
        m_pVal2SpinBox->blockSignals( true );
        m_pVal2SpinBox->setMinimum( m_nKneepoint1Val+1 );
        m_pVal2SpinBox->blockSignals( false );
        m_pVal2Slider->blockSignals( true );
        m_pVal2Slider->setMinimum( m_nKneepoint1Val+1 );
        m_pVal2Slider->blockSignals( false );
    }

    int nVal2Max = m_nKneepointCount>2? m_nKneepoint3Val-1 : ( 1<<16 ) - 1;
    if( m_pVal2SpinBox->maximum() != m_nKneepoint3Val-1 )
    {
        m_pVal2SpinBox->blockSignals( true );
        m_pVal2SpinBox->setMaximum( nVal2Max );
        m_pVal2SpinBox->blockSignals( false );
        m_pVal2Slider->blockSignals( true );
        m_pVal2Slider->setMaximum( nVal2Max );
        m_pVal2Slider->blockSignals( false );
    }

    if ( m_nKneepointCount < 3 )
    {
        return;
    }

    if( m_pVal3SpinBox->minimum() != m_nKneepoint2Val+1 )
    {
        m_pVal3SpinBox->blockSignals( true );
        m_pVal3SpinBox->setMinimum( m_nKneepoint2Val+1 );
        m_pVal3SpinBox->blockSignals( false );
        m_pVal3Slider->blockSignals( true );
        m_pVal3Slider->setMinimum( m_nKneepoint2Val+1 );
        m_pVal3Slider->blockSignals( false );
    }
}
