/** \file hsnrcontrol.cpp
 *  \brief This file contains the implementation of the feature control class HsnrControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "hsnrcontrol.h"

HsnrControl::HsnrControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_HsnrControl()
{
    setupUi ( this );

    m_pSpinBox->setMinimum( 1 );
    m_pSpinBox->setMaximum( 99999 );

    connect( m_pSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onValueChange(int) ) );
    connect( m_pEnableCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onEnableClicked(bool) ) );
}


HsnrControl::~HsnrControl()
{
}


void HsnrControl::setCamera( Dc1394Camera*      pCamera, 
                             AcquisitionThread* /*pThread*/ )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    /* check feature support */
    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->HSNR )
    {
        /* HSNR is not supported */
        setEnabled(false);
        return;
    }

    setEnabled(true);
    pollStateFromCamera();
}


void HsnrControl::pollStateFromCamera()
{
    dc1394error_t eRet;
    uint32_t nVal;
    dc1394bool_t eActive;
    eRet = dc1394_avt_get_hsnr( m_pCamObject->psCamInternal(), &eActive, &nVal);
    LOGFCALL_LIBDC ( "dc1394_avt_get_hsnr", eRet );

    m_nCurrentGrabCount = (int) nVal;
    m_bActive = ( DC1394_TRUE == eActive );

    m_pSpinBox->setValue( m_nCurrentGrabCount );
    m_pEnableCheckBox->setChecked( m_bActive );
}


void HsnrControl::onStateChangedByTwinControl(void)
{
    HsnrControl* pSender = qobject_cast<HsnrControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nCurrentGrabCount = pSender->m_nCurrentGrabCount;
    m_bActive = pSender->m_bActive;
    m_pSpinBox->setValue( m_nCurrentGrabCount );
    m_pEnableCheckBox->setChecked( m_bActive );
}


void HsnrControl::onValueChange( int nIndex )
{
    if( nIndex != m_nCurrentGrabCount )
    {
        m_nCurrentGrabCount = nIndex;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_hsnr( m_pCamObject->psCamInternal(), 
                                    m_bActive? DC1394_TRUE : DC1394_FALSE,
                                    (uint32_t) m_nCurrentGrabCount );
        LOGFCALL_LIBDC ( "dc1394_avt_set_hsnr", eRet );

        emit featureStateChanged();
        emit HsnrSettingsChange( m_bActive, m_nCurrentGrabCount );
    }
}


void HsnrControl::onEnableClicked( bool bChecked )
{
    if( bChecked != m_bActive )
    {
        m_bActive = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_hsnr( m_pCamObject->psCamInternal(), 
                                    m_bActive? DC1394_TRUE : DC1394_FALSE,
                                    (uint32_t) m_nCurrentGrabCount );
        LOGFCALL_LIBDC ( "dc1394_avt_set_hsnr", eRet );

        emit featureStateChanged();
        emit HsnrSettingsChange( m_bActive, m_nCurrentGrabCount );
    }
}
