/** \file inputdebouncecontrol.cpp
 *  \brief This file contains the implementation of the feature control class InputDebounceControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "inputdebouncecontrol.h"

InputDebounceControl::InputDebounceControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_InputDebounceControl()
{
    setupUi ( this );

    // connect controls
    connect( m_pInp1SpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onValueChange_1(int) ) );
    connect( m_pInp2SpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onValueChange_2(int) ) );
    connect( m_pInp3SpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onValueChange_3(int) ) );
    connect( m_pInp4SpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onValueChange_4(int) ) );
}


InputDebounceControl::~InputDebounceControl()
{
}


void InputDebounceControl::setCamera( Dc1394Camera*      pCamera,
                                     AcquisitionThread* /*pThread*/ )
{
    dc1394error_t eRet;
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    /* check feature support */
    m_bInput1Supported = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->InputDebounce_1 );
    m_bInput2Supported = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->InputDebounce_2 );
    m_bInput3Supported = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->InputDebounce_3 );
    m_bInput4Supported = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->InputDebounce_4 );

    if ( !m_bInput1Supported &&
         !m_bInput2Supported &&
         !m_bInput3Supported &&
         !m_bInput4Supported )
    {
        /* Input Debounce is not supported at all */
        setEnabled(false);
        return;
    }

    m_pInp1Slider->setEnabled(m_bInput1Supported);
    m_pInp1SpinBox->setEnabled(m_bInput1Supported);
    m_pInp2Slider->setEnabled(m_bInput2Supported);
    m_pInp2SpinBox->setEnabled(m_bInput2Supported);
    m_pInp3Slider->setEnabled(m_bInput3Supported);
    m_pInp3SpinBox->setEnabled(m_bInput3Supported);
    m_pInp4Slider->setEnabled(m_bInput4Supported);
    m_pInp4SpinBox->setEnabled(m_bInput4Supported);

    /* set ranges */
    uint32_t nMin, nMax;
    if(m_bInput1Supported)
    {
        eRet = dc1394_avt_get_io_inp_debounce_info( m_pCamObject->psCamInternal(),
                                                    DC1394_AVT_IO_INP_DEBOUNCE_CTRL1,
                                                    &nMin,
                                                    &nMax );
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_inp_debounce_info", eRet );

        m_pInp1Slider->blockSignals(true);
        m_pInp1SpinBox->blockSignals(true);
        m_pInp1Slider->setMinimum((int) nMin);
        m_pInp1Slider->setMaximum((int) nMax);
        m_pInp1SpinBox->setMinimum((int) nMin);
        m_pInp1SpinBox->setMaximum((int) nMax);
        m_pInp1Slider->blockSignals(false);
        m_pInp1SpinBox->blockSignals(false);
    }
    if(m_bInput2Supported)
    {
        eRet = dc1394_avt_get_io_inp_debounce_info( m_pCamObject->psCamInternal(),
                                                    DC1394_AVT_IO_INP_DEBOUNCE_CTRL2,
                                                    &nMin,
                                                    &nMax );
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_inp_debounce_info", eRet );

        m_pInp2Slider->blockSignals(true);
        m_pInp2SpinBox->blockSignals(true);
        m_pInp2Slider->setMinimum((int) nMin);
        m_pInp2Slider->setMaximum((int) nMax);
        m_pInp2SpinBox->setMinimum((int) nMin);
        m_pInp2SpinBox->setMaximum((int) nMax);
        m_pInp2Slider->blockSignals(false);
        m_pInp2SpinBox->blockSignals(false);
    }
    if(m_bInput3Supported)
    {
        eRet = dc1394_avt_get_io_inp_debounce_info( m_pCamObject->psCamInternal(),
                                                    DC1394_AVT_IO_INP_DEBOUNCE_CTRL3,
                                                    &nMin,
                                                    &nMax );
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_inp_debounce_info", eRet );

        m_pInp3Slider->blockSignals(true);
        m_pInp3SpinBox->blockSignals(true);
        m_pInp3Slider->setMinimum((int) nMin);
        m_pInp3Slider->setMaximum((int) nMax);
        m_pInp3SpinBox->setMinimum((int) nMin);
        m_pInp3SpinBox->setMaximum((int) nMax);
        m_pInp3Slider->blockSignals(false);
        m_pInp3SpinBox->blockSignals(false);
    }
    if(m_bInput4Supported)
    {
        eRet = dc1394_avt_get_io_inp_debounce_info( m_pCamObject->psCamInternal(),
                                                    DC1394_AVT_IO_INP_DEBOUNCE_CTRL4,
                                                    &nMin,
                                                    &nMax );
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_inp_debounce_info", eRet );

        m_pInp4Slider->blockSignals(true);
        m_pInp4SpinBox->blockSignals(true);
        m_pInp4Slider->setMinimum((int) nMin);
        m_pInp4Slider->setMaximum((int) nMax);
        m_pInp4SpinBox->setMinimum((int) nMin);
        m_pInp4SpinBox->setMaximum((int) nMax);
        m_pInp4Slider->blockSignals(false);
        m_pInp4SpinBox->blockSignals(false);
    }

    m_bRefreshRequired = false;
    pollStateFromCamera();
    setEnabled(true);
}


void InputDebounceControl::pollStateFromCamera()
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    uint32_t nVal;
    if( m_bInput1Supported )
    {
        eRet = dc1394_avt_get_io_inp_debounce( m_pCamObject->psCamInternal(),
                                               DC1394_AVT_IO_INP_DEBOUNCE_CTRL1,
                                               &nVal );
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_inp_debounce", eRet );
        m_nValue_1 = (int) nVal;
        m_pInp1SpinBox->setValue(m_nValue_1);
    }
    if( m_bInput2Supported )
    {
        eRet = dc1394_avt_get_io_inp_debounce( m_pCamObject->psCamInternal(),
                                               DC1394_AVT_IO_INP_DEBOUNCE_CTRL2,
                                               &nVal );
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_inp_debounce", eRet );
        m_nValue_2 = (int) nVal;
        m_pInp2SpinBox->setValue(m_nValue_2);
    }
    if( m_bInput3Supported )
    {
        eRet = dc1394_avt_get_io_inp_debounce( m_pCamObject->psCamInternal(),
                                               DC1394_AVT_IO_INP_DEBOUNCE_CTRL3,
                                               &nVal );
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_inp_debounce", eRet );
        m_nValue_3 = (int) nVal;
        m_pInp3SpinBox->setValue(m_nValue_3);
    }
    if( m_bInput4Supported )
    {
        eRet = dc1394_avt_get_io_inp_debounce( m_pCamObject->psCamInternal(),
                                               DC1394_AVT_IO_INP_DEBOUNCE_CTRL4,
                                               &nVal );
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_inp_debounce", eRet );
        m_nValue_4 = (int) nVal;
        m_pInp4SpinBox->setValue(m_nValue_4);
    }
}


void InputDebounceControl::onStateChangedByTwinControl(void)
{
    InputDebounceControl* pSender = qobject_cast<InputDebounceControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nValue_1 = pSender->m_nValue_1;
    m_nValue_2 = pSender->m_nValue_2;
    m_nValue_3 = pSender->m_nValue_3;
    m_nValue_4 = pSender->m_nValue_4;

    m_pInp1SpinBox->setValue(m_nValue_1);
    m_pInp2SpinBox->setValue(m_nValue_2);
    m_pInp3SpinBox->setValue(m_nValue_3);
    m_pInp4SpinBox->setValue(m_nValue_4);
}


void InputDebounceControl::onValueChange_1(int nValue)
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nValue_1 != nValue )
    {
        m_nValue_1 = nValue;

        /* configure camera */
        eRet = dc1394_avt_set_io_inp_debounce( m_pCamObject->psCamInternal(),
                                               DC1394_AVT_IO_INP_DEBOUNCE_CTRL1,
                                               m_nValue_1 );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io_inp_debounce", eRet );
    }
    emit featureStateChanged();
}


void InputDebounceControl::onValueChange_2(int nValue)
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nValue_2 != nValue )
    {
        m_nValue_2 = nValue;

        /* configure camera */
        eRet = dc1394_avt_set_io_inp_debounce( m_pCamObject->psCamInternal(),
                                               DC1394_AVT_IO_INP_DEBOUNCE_CTRL2,
                                               m_nValue_2 );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io_inp_debounce", eRet );
    }
    emit featureStateChanged();
}


void InputDebounceControl::onValueChange_3(int nValue)
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nValue_3 != nValue )
    {
        m_nValue_3 = nValue;

        /* configure camera */
        eRet = dc1394_avt_set_io_inp_debounce( m_pCamObject->psCamInternal(),
                                               DC1394_AVT_IO_INP_DEBOUNCE_CTRL3,
                                               m_nValue_3 );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io_inp_debounce", eRet );
    }
    emit featureStateChanged();
}


void InputDebounceControl::onValueChange_4(int nValue)
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nValue_4 != nValue )
    {
        m_nValue_4 = nValue;

        /* configure camera */
        eRet = dc1394_avt_set_io_inp_debounce( m_pCamObject->psCamInternal(),
                                               DC1394_AVT_IO_INP_DEBOUNCE_CTRL4,
                                               m_nValue_4 );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io_inp_debounce", eRet );
    }
    emit featureStateChanged();
}

