/** \file maxisosizecontrol.cpp
 *  \brief This file contains the implementation of the feature control class MaxIsoSizeControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <dc1394/vendor/avt.h>

#include "maxisosizecontrol.h"

MaxIsoSizeControl::MaxIsoSizeControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_MaxIsoSizeControl()
{
    setupUi ( this );

    // connect controls
    connect( m_pModeComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onModeSwitched(int)) );
    connect( m_pEnableCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onEnableClicked(bool)) );
    connect( m_pSpinBox, SIGNAL( valueChanged(int)), this, SLOT(onValueChanged(int)) );
    connect( m_pMaxPushButton, SIGNAL(clicked()), this, SLOT(onMaxClicked()) );
}


MaxIsoSizeControl::~MaxIsoSizeControl()
{
}


void MaxIsoSizeControl::setCamera( Dc1394Camera*      pCamera,
                                   AcquisitionThread* /*pThread*/ )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    /* check feature support */
    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->MaxIsoSize_S400 &&
         DC1394_FALSE == pCamera->smartFeatureInquiry()->MaxIsoSize_S800 )
    {
        /* max iso size feature is not supported */
        setEnabled(false);
        return;
    }

    m_pModeComboBox->clear();
    if ( DC1394_TRUE == pCamera->smartFeatureInquiry()->MaxIsoSize_S400 )
    {
        m_pModeComboBox->addItem( "S400", QVariant( (int) E_MODE_S400 ) );
    }
    if ( DC1394_TRUE == pCamera->smartFeatureInquiry()->MaxIsoSize_S800 )
    {
        m_pModeComboBox->addItem( "S800", QVariant( (int) E_MODE_S800 ) );
    }

    m_eCurrentMode = (E_MODE) m_pModeComboBox->itemData(m_pModeComboBox->currentIndex()).toInt();
    pollStateFromCamera();
    setEnabled(true);
}

void MaxIsoSizeControl::onStateChangedByTwinControl()
{
    MaxIsoSizeControl* pSender = qobject_cast<MaxIsoSizeControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nValue = pSender->m_nValue;
    m_bEnabled = pSender->m_bEnabled;
    m_eCurrentMode = pSender->m_eCurrentMode;

    m_pSpinBox->blockSignals( true );
    m_pSpinBox->setValue( m_nValue );
    m_pSpinBox->blockSignals( false );

    m_pModeComboBox->blockSignals( true );
    m_pModeComboBox->setCurrentIndex( (int) m_eCurrentMode );
    m_pModeComboBox->blockSignals( false );

    m_pEnableCheckBox->blockSignals(true);
    m_pEnableCheckBox->setChecked(m_bEnabled);
    m_pEnableCheckBox->blockSignals(false);
}

void MaxIsoSizeControl::pollStateFromCamera()
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    switch( m_eCurrentMode )
    {
        case E_MODE_S400:
            m_pSpinBox->setMaximum(5500);
            break;
        case E_MODE_S800:
            m_pSpinBox->setMaximum(11000);
            break;
        default:
            break;
    }

    /* get settings for current mode*/#
    dc1394error_t   eRet;
    uint32_t        nVal;
    dc1394switch_t  eOnOff;
    eRet = dc1394_avt_get_max_iso_size( m_pCamObject->psCamInternal(),
                                        (uint32_t) m_eCurrentMode,
                                        &eOnOff,
                                        &nVal);
    LOGFCALL_LIBDC ( "dc1394_avt_get_max_iso_size", eRet );

    if( m_bEnabled != (eOnOff==DC1394_ON) )
    {
        m_bEnabled = (eOnOff==DC1394_ON);
        m_pEnableCheckBox->setChecked(m_bEnabled);
    }
    
    if( m_nValue != (int) nVal )
    {
        m_nValue = (int) nVal;
        m_pSpinBox->setValue( m_nValue );
    }
}

void MaxIsoSizeControl::onEnableClicked( bool bEnable )
{
    dc1394error_t   eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }
    if(bEnable!=m_bEnabled)
    {
        m_bEnabled = bEnable;
        eRet = dc1394_avt_set_max_iso_size( m_pCamObject->psCamInternal(),
                                            (uint32_t) m_eCurrentMode,
                                            m_bEnabled? DC1394_ON : DC1394_OFF,
                                            DC1394_OFF,
                                            m_nValue );
        LOGFCALL_LIBDC ( "dc1394_avt_set_max_iso_size", eRet );

        emit MaxIsoSizeChange();
    }
}

void MaxIsoSizeControl::onModeSwitched( int nIndex )
 {
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_eCurrentMode != (E_MODE) m_pModeComboBox->itemData(nIndex).toInt() )
    {
        m_eCurrentMode = (E_MODE) m_pModeComboBox->itemData(nIndex).toInt();
        pollStateFromCamera();
    }
 }

 void MaxIsoSizeControl::onValueChanged( int nValue )
 {
    dc1394error_t   eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }
    if(nValue!=m_nValue)
    {
        m_nValue = nValue;
        eRet = dc1394_avt_set_max_iso_size( m_pCamObject->psCamInternal(),
                                            (uint32_t) m_eCurrentMode,
                                            m_bEnabled? DC1394_ON : DC1394_OFF,
                                            DC1394_OFF,
                                            m_nValue );
        LOGFCALL_LIBDC ( "dc1394_avt_set_max_iso_size", eRet );

        emit MaxIsoSizeChange();
    }
 }

 void MaxIsoSizeControl::onMaxClicked()
 {
    dc1394error_t   eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    eRet = dc1394_avt_set_max_iso_size( m_pCamObject->psCamInternal(),
                                        (uint32_t) m_eCurrentMode,
                                        m_bEnabled? DC1394_ON : DC1394_OFF,
                                        DC1394_ON,
                                        m_nValue );
    LOGFCALL_LIBDC ( "dc1394_avt_set_max_iso_size", eRet );

    /* value has been set to maximum, refresh feature state */
    usleep(50000);
    pollStateFromCamera();

    emit MaxIsoSizeChange();
 }
