/** \file packetsizecontrol.cpp
 *  \brief This file contains the implementation of the feature control class PacketSizeControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <dc1394/vendor/avt.h>

#include "packetsizecontrol.h"

PacketSizeControl::PacketSizeControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_PacketSizeControl()
{
    setupUi ( this );

    // connect controls
    connect( m_pSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onValueChange(int) ) );
    connect( m_pAutoCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onAutoClicked(bool) ) );
}


PacketSizeControl::~PacketSizeControl()
{
}


void PacketSizeControl::setCamera( Dc1394Camera*      pCamera, 
                                   AcquisitionThread* /*pThread*/ )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    /* todo: auto settings need to be stored somewhere on the host.
       for now, we activate auto when a camera is selected */
    m_bAutoActive = true;
    m_pAutoCheckBox->blockSignals( true );
    m_pAutoCheckBox->setChecked( true );
    m_pAutoCheckBox->blockSignals( false );

    onVideoModeSwitched_autoconnect();
}

void PacketSizeControl::setAcquisitionState( bool bActive )
{
    m_pSpinBox->setEnabled( !bActive && !m_bAutoActive );
    m_pSlider->setEnabled( !bActive && !m_bAutoActive );
    m_pAutoCheckBox->setEnabled( !bActive );
}

void PacketSizeControl::onImageSizeChanged_autoconnect( int /*w*/, int /*h*/ )
{
    getValidRange();
    pollStateFromCamera();

    if( m_bAutoActive )
    {
        setToRecommendedSize();
    }
}


void PacketSizeControl::onStateChangedByTwinControl()
{
    PacketSizeControl* pSender = qobject_cast<PacketSizeControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nValue = pSender->m_nValue;
    m_bAutoActive = pSender->m_bAutoActive;

    m_pSpinBox->blockSignals( true );
    m_pSpinBox->setValue( m_nValue );
    m_pSpinBox->blockSignals( false );
    m_pSlider->setValue( m_nValue );

    m_pAutoCheckBox->blockSignals( true );
    m_pAutoCheckBox->setCheckState( m_bAutoActive ? Qt::Checked : Qt::Unchecked );
    m_pAutoCheckBox->blockSignals( false );

    m_pSlider->setEnabled( !m_bAutoActive );
    m_pSpinBox->setEnabled( !m_bAutoActive );
}


void PacketSizeControl::onColorCodingChange_autoconnect( dc1394color_coding_t /*eColorCoding*/ )
{
    getValidRange();
    pollStateFromCamera();

    if( m_bAutoActive )
    {
        setToRecommendedSize();
    }
}


void PacketSizeControl::onMaxIsoSizeChange_autoconnect()
{
    onVideoModeSwitched_autoconnect();
}

void PacketSizeControl::onVideoModeSwitched_autoconnect()
{
    if ( NULL == m_pCamObject )
    {
        return;
    }

    if ( m_pCamObject->getVideoMode() < DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        // not a format 7 mode, fixed packet size
        setEnabled( false );
        return;
    }

    getValidRange();
    pollStateFromCamera();

    if( m_bAutoActive )
    {
        setToRecommendedSize();
    }

    m_pSlider->setEnabled( !m_bAutoActive );
    m_pSpinBox->setEnabled( !m_bAutoActive );
    setEnabled(true);
}


void PacketSizeControl::onIsoSpeedChanged_autoconnect()
{
    if ( NULL == m_pCamObject )
    {
        return;
    }

    if ( m_pCamObject->getVideoMode() < DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        // not a format 7 mode, fixed packet size
        setEnabled( false );
        return;
    }

    getValidRange();
    pollStateFromCamera();

    if( m_bAutoActive )
    {
        setToRecommendedSize();
    }

    setEnabled(true);
}


void PacketSizeControl::getValidRange()
{
    uint32_t nUnitBytes;
    uint32_t nMaxBytes;
    dc1394error_t eRet;
    eRet = dc1394_format7_get_packet_parameters( m_pCamObject->psCamInternal(),
                                                 m_pCamObject->getVideoMode(),
                                                 &nUnitBytes,
                                                 &nMaxBytes );
    LOGFCALL_LIBDC ( "dc1394_format7_get_packet_parameters", eRet );

    m_nUnitBytes = (int) nUnitBytes;
    m_nMaxBytes = (int) nMaxBytes;

    /* set min/max */
    m_pSlider->setMinimum( (int) nUnitBytes );
    m_pSlider->setSingleStep( (int) nUnitBytes );
    m_pSlider->setMaximum( (int) nMaxBytes );
    m_pSpinBox->setMinimum( (int) nUnitBytes );
    m_pSlider->setSingleStep( (int) nUnitBytes );
    m_pSpinBox->setMaximum( (int) nMaxBytes );
    m_pSpinBox->setSingleStep( (int) nUnitBytes );
}

void PacketSizeControl::pollStateFromCamera()
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if ( m_pCamObject->getVideoMode() < DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        // not a format 7 mode, fixed packet size
        setEnabled( false );
        return;
    }
    setEnabled(true);

    /* get current value */#
    dc1394error_t eRet;
    uint32_t nVal;
    eRet = dc1394_format7_get_packet_size( m_pCamObject->psCamInternal(),
                                           m_pCamObject->getVideoMode(),
                                           &nVal);
    LOGFCALL_LIBDC ( "dc1394_format7_get_packet_size", eRet );
    
    if( m_nValue != (int) nVal )
    {
        m_nValue = (int) nVal;
        m_pSpinBox->setValue( m_nValue );
    }
}


void PacketSizeControl::onValueChange(int nValue)
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nValue != nValue )
    {
        m_nValue = nValue;

        if ( 0!= m_nValue % m_nUnitBytes )
        {
            m_nValue -= m_nValue % m_nUnitBytes;
            m_nValue = std::max( m_nUnitBytes, m_nValue );
            m_pSpinBox->setValue( m_nValue );
        }

        /* configure camera */
        dc1394error_t eRet  = dc1394_format7_set_packet_size( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), m_nValue);
        LOGFCALL_LIBDC ( "dc1394_format7_set_packet_size", eRet );
    }
    emit featureStateChanged();
}

void PacketSizeControl::onAutoClicked( bool bChecked )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_bAutoActive != bChecked )
    {
        m_bAutoActive = bChecked;

        m_pSlider->setEnabled( !m_bAutoActive );
        m_pSpinBox->setEnabled( !m_bAutoActive );

        if( m_bAutoActive )
        {
            setToRecommendedSize();
        }
    }
    emit featureStateChanged();
}

void PacketSizeControl::setToRecommendedSize()
{
    dc1394error_t eRet;
    uint32_t nSize;
    eRet = dc1394_format7_get_recommended_packet_size( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nSize);
    LOGFCALL_LIBDC ( "dc1394_format7_get_recommended_packet_size", eRet );
    if ( 0 != nSize )
    {
        m_nValue = nSize;
        m_pSpinBox->blockSignals( true );
        m_pSpinBox->setValue( (int) m_nValue );
        m_pSpinBox->blockSignals( false );
        m_pSlider->setValue( (int) m_nValue );

        eRet = dc1394_format7_set_packet_size( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), m_nValue);
        LOGFCALL_LIBDC ( "dc1394_format7_set_packet_size", eRet );
    }
}

