/** \file pwmoutcontrol.cpp
 *  \brief This file contains the implementation of the feature control class PwmOutControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "pwmoutcontrol.h"

PwmOutControl::PwmOutControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_PwmOutControl()
{
    setupUi ( this );

    // connect controls
    connect( m_pOutp1Period, SIGNAL( valueChanged(int) ), this, SLOT( onPeriodChange_1(int)) );
    connect( m_pOutp2Period, SIGNAL( valueChanged(int) ), this, SLOT( onPeriodChange_2(int)) );
    connect( m_pOutp3Period, SIGNAL( valueChanged(int) ), this, SLOT( onPeriodChange_3(int)) );
    connect( m_pOutp4Period, SIGNAL( valueChanged(int) ), this, SLOT( onPeriodChange_4(int)) );
    connect( m_pOutp1Width, SIGNAL( valueChanged(int) ), this, SLOT( onWidthChange_1(int) ) );
    connect( m_pOutp2Width, SIGNAL( valueChanged(int) ), this, SLOT( onWidthChange_2(int) ) );
    connect( m_pOutp3Width, SIGNAL( valueChanged(int) ), this, SLOT( onWidthChange_3(int) ) );
    connect( m_pOutp4Width, SIGNAL( valueChanged(int) ), this, SLOT( onWidthChange_4(int) ) );
}


PwmOutControl::~PwmOutControl()
{
}


void PwmOutControl::setCamera( Dc1394Camera*      pCamera,
                               AcquisitionThread* /*pThread*/ )
{
    dc1394error_t eRet;
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    /* check feature support */
    m_bPwmOut1Support = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->Output_1_PWM );
    m_bPwmOut2Support = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->Output_2_PWM );
    m_bPwmOut3Support = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->Output_3_PWM );
    m_bPwmOut4Support = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->Output_4_PWM );

    if ( !m_bPwmOut1Support &&
         !m_bPwmOut2Support &&
         !m_bPwmOut3Support &&
         !m_bPwmOut4Support )
    {
        /* PWM Output is not supported at all */
        setEnabled(false);
        return;
    }

    m_pOutp1Period->setEnabled(m_bPwmOut1Support);
    m_pOutp1Width->setEnabled(m_bPwmOut1Support);
    m_pOutp2Period->setEnabled(m_bPwmOut2Support);
    m_pOutp2Width->setEnabled(m_bPwmOut2Support);
    m_pOutp3Period->setEnabled(m_bPwmOut3Support);
    m_pOutp3Width->setEnabled(m_bPwmOut3Support);
    m_pOutp4Period->setEnabled(m_bPwmOut4Support);
    m_pOutp4Width->setEnabled(m_bPwmOut4Support);

    /* set ranges */
    uint32_t nMin;

    if(m_bPwmOut1Support)
    {
        eRet = dc1394_avt_get_io_pwmout_info(m_pCamObject->psCamInternal(),
                                             DC1394_AVT_IO_OUTP_PWM_CTRL1,
                                             &nMin);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_pwmout_info", eRet );
        m_pOutp1Period->setMinimum((int) nMin);
        m_pOutp1Width->setMinimum((int) nMin);
        m_pOutp1Period->setMaximum((1<<16)-1);
        m_pOutp1Width->setMaximum((1<<16)-1);
    }

    if(m_bPwmOut2Support)
    {
        eRet = dc1394_avt_get_io_pwmout_info(m_pCamObject->psCamInternal(),
                                             DC1394_AVT_IO_OUTP_PWM_CTRL2,
                                             &nMin);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_pwmout_info", eRet );
        m_pOutp2Period->setMinimum((int) nMin);
        m_pOutp2Width->setMinimum((int) nMin);
        m_pOutp2Period->setMaximum((1<<16)-1);
        m_pOutp2Width->setMaximum((1<<16)-1);
    }

    if(m_bPwmOut3Support)
    {
        eRet = dc1394_avt_get_io_pwmout_info(m_pCamObject->psCamInternal(),
                                             DC1394_AVT_IO_OUTP_PWM_CTRL3,
                                             &nMin);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_pwmout_info", eRet );
        m_pOutp3Period->setMinimum((int) nMin);
        m_pOutp3Width->setMinimum((int) nMin);
        m_pOutp3Period->setMaximum((1<<16)-1);
        m_pOutp3Width->setMaximum((1<<16)-1);
    }

    if(m_bPwmOut4Support)
    {
        eRet = dc1394_avt_get_io_pwmout_info(m_pCamObject->psCamInternal(),
                                             DC1394_AVT_IO_OUTP_PWM_CTRL4,
                                             &nMin);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_pwmout_info", eRet );
        m_pOutp4Period->setMinimum((int) nMin);
        m_pOutp4Width->setMinimum((int) nMin);
        m_pOutp4Period->setMaximum((1<<16)-1);
        m_pOutp4Width->setMaximum((1<<16)-1);
    }

    m_bRefreshRequired = false;
    pollStateFromCamera();
    setEnabled(true);
}


void PwmOutControl::pollStateFromCamera()
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    uint32_t nPeriod, nWidth;
    if( m_bPwmOut1Support )
    {
        eRet = dc1394_avt_get_io_pwmout(m_pCamObject->psCamInternal(),
                                        DC1394_AVT_IO_OUTP_PWM_CTRL1,
                                        &nPeriod,
                                        &nWidth);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_pwmout", eRet );
        m_nPeriod_1 = (int) nPeriod;
        m_nWidth_1 = (int) nWidth;
        m_pOutp1Period->setValue(m_nPeriod_1);
        m_pOutp1Width->setValue(m_nWidth_1);
    }
    if( m_bPwmOut2Support )
    {
        eRet = dc1394_avt_get_io_pwmout(m_pCamObject->psCamInternal(),
                                        DC1394_AVT_IO_OUTP_PWM_CTRL2,
                                        &nPeriod,
                                        &nWidth);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_pwmout", eRet );
        m_nPeriod_2 = (int) nPeriod;
        m_nWidth_2 = (int) nWidth;
        m_pOutp2Period->setValue(m_nPeriod_2);
        m_pOutp2Width->setValue(m_nWidth_2);
    }
    if( m_bPwmOut3Support )
    {
        eRet = dc1394_avt_get_io_pwmout(m_pCamObject->psCamInternal(),
                                        DC1394_AVT_IO_OUTP_PWM_CTRL3,
                                        &nPeriod,
                                        &nWidth);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_pwmout", eRet );
        m_nPeriod_3 = (int) nPeriod;
        m_nWidth_3 = (int) nWidth;
        m_pOutp3Period->setValue(m_nPeriod_3);
        m_pOutp3Width->setValue(m_nWidth_3);
    }
    if( m_bPwmOut4Support )
    {
        eRet = dc1394_avt_get_io_pwmout(m_pCamObject->psCamInternal(),
                                        DC1394_AVT_IO_OUTP_PWM_CTRL4,
                                        &nPeriod,
                                        &nWidth);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io_pwmout", eRet );
        m_nPeriod_4 = (int) nPeriod;
        m_nWidth_4 = (int) nWidth;
        m_pOutp4Period->setValue(m_nPeriod_4);
        m_pOutp4Width->setValue(m_nWidth_4);
    }
    adjustWidthRanges();
}

void PwmOutControl::adjustWidthRanges()
{
    m_pOutp1Width->setMaximum(m_pOutp1Period->value()-1);
    m_pOutp2Width->setMaximum(m_pOutp2Period->value()-1);
    m_pOutp3Width->setMaximum(m_pOutp3Period->value()-1);
    m_pOutp4Width->setMaximum(m_pOutp4Period->value()-1);
}

void PwmOutControl::onStateChangedByTwinControl(void)
{
    PwmOutControl* pSender = qobject_cast<PwmOutControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nPeriod_1 = pSender->m_nPeriod_1;
    m_nPeriod_2 = pSender->m_nPeriod_2;
    m_nPeriod_3 = pSender->m_nPeriod_3;
    m_nPeriod_4 = pSender->m_nPeriod_4;

    m_nWidth_1 = pSender->m_nWidth_1;
    m_nWidth_2 = pSender->m_nWidth_2;
    m_nWidth_3 = pSender->m_nWidth_3;
    m_nWidth_4 = pSender->m_nWidth_4;

    m_pOutp1Period->setValue(m_nPeriod_1);
    m_pOutp2Period->setValue(m_nPeriod_2);
    m_pOutp3Period->setValue(m_nPeriod_3);
    m_pOutp4Period->setValue(m_nPeriod_4);
    m_pOutp1Width->setValue(m_nWidth_1);
    m_pOutp2Width->setValue(m_nWidth_2);
    m_pOutp3Width->setValue(m_nWidth_3);
    m_pOutp4Width->setValue(m_nWidth_4);

    adjustWidthRanges();
}

void PwmOutControl::onPeriodChange_1( int nValue )
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nPeriod_1 != nValue )
    {
        m_nPeriod_1 = nValue;

        /* configure camera */
        eRet = dc1394_avt_set_io_pwmout(m_pCamObject->psCamInternal(),
                                        DC1394_AVT_IO_OUTP_PWM_CTRL1,
                                        (uint32_t) m_nPeriod_1,
                                        (uint32_t) m_nWidth_1);
        LOGFCALL_LIBDC ( "dc1394_avt_set_io_pwmout", eRet );
    }
    adjustWidthRanges();
}

void PwmOutControl::onPeriodChange_2( int nValue )
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nPeriod_2 != nValue )
    {
        m_nPeriod_2 = nValue;

        /* configure camera */
        eRet = dc1394_avt_set_io_pwmout(m_pCamObject->psCamInternal(),
                                        DC1394_AVT_IO_OUTP_PWM_CTRL2,
                                        (uint32_t) m_nPeriod_2,
                                        (uint32_t) m_nWidth_2);
        LOGFCALL_LIBDC ( "dc1394_avt_set_io_pwmout", eRet );
    }
    adjustWidthRanges();
}

void PwmOutControl::onPeriodChange_3( int nValue )
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nPeriod_3 != nValue )
    {
        m_nPeriod_3 = nValue;

        /* configure camera */
        eRet = dc1394_avt_set_io_pwmout(m_pCamObject->psCamInternal(),
                                        DC1394_AVT_IO_OUTP_PWM_CTRL3,
                                        (uint32_t) m_nPeriod_3,
                                        (uint32_t) m_nWidth_3);
        LOGFCALL_LIBDC ( "dc1394_avt_set_io_pwmout", eRet );
    }
    adjustWidthRanges();
}

void PwmOutControl::onPeriodChange_4( int nValue )
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nPeriod_4 != nValue )
    {
        m_nPeriod_4 = nValue;

        /* configure camera */
        eRet = dc1394_avt_set_io_pwmout(m_pCamObject->psCamInternal(),
                                        DC1394_AVT_IO_OUTP_PWM_CTRL4,
                                        (uint32_t) m_nPeriod_4,
                                        (uint32_t) m_nWidth_4);
        LOGFCALL_LIBDC ( "dc1394_avt_set_io_pwmout", eRet );
    }
    adjustWidthRanges();
}

void PwmOutControl::onWidthChange_1( int nValue )
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nWidth_1 != nValue )
    {
        m_nWidth_1 = nValue;

        /* configure camera */
        eRet = dc1394_avt_set_io_pwmout(m_pCamObject->psCamInternal(),
                                        DC1394_AVT_IO_OUTP_PWM_CTRL1,
                                        (uint32_t) m_nPeriod_1,
                                        (uint32_t) m_nWidth_1);
        LOGFCALL_LIBDC ( "dc1394_avt_set_io_pwmout", eRet );
    }
}

void PwmOutControl::onWidthChange_2( int nValue )
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nWidth_2 != nValue )
    {
        m_nWidth_2 = nValue;

        /* configure camera */
        eRet = dc1394_avt_set_io_pwmout(m_pCamObject->psCamInternal(),
                                        DC1394_AVT_IO_OUTP_PWM_CTRL2,
                                        (uint32_t) m_nPeriod_2,
                                        (uint32_t) m_nWidth_2);
        LOGFCALL_LIBDC ( "dc1394_avt_set_io_pwmout", eRet );
    }
}

void PwmOutControl::onWidthChange_3( int nValue )
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nWidth_3 != nValue )
    {
        m_nWidth_3 = nValue;

        /* configure camera */
        eRet = dc1394_avt_set_io_pwmout(m_pCamObject->psCamInternal(),
                                        DC1394_AVT_IO_OUTP_PWM_CTRL3,
                                        (uint32_t) m_nPeriod_3,
                                        (uint32_t) m_nWidth_3);
        LOGFCALL_LIBDC ( "dc1394_avt_set_io_pwmout", eRet );
    }
}

void PwmOutControl::onWidthChange_4( int nValue )
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nWidth_4 != nValue )
    {
        m_nWidth_4 = nValue;

        /* configure camera */
        eRet = dc1394_avt_set_io_pwmout(m_pCamObject->psCamInternal(),
                                        DC1394_AVT_IO_OUTP_PWM_CTRL4,
                                        (uint32_t) m_nPeriod_4,
                                        (uint32_t) m_nWidth_4);
        LOGFCALL_LIBDC ( "dc1394_avt_set_io_pwmout", eRet );
    }
}

