/** \file shadingcontrol.cpp
 *  \brief This file contains the implementation of the feature control class ShadingControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "shadingcontrol.h"

#include <QFileDialog>
#include <QMessageBox>

ShadingControl::ShadingControl ( QWidget* pParent )
        : AbstractFeatureControl ( pParent )
        , Ui_ShadingControl()
{
    setupUi ( this );

    m_pSpinBox->setMinimum( 1 );
    m_pSpinBox->setMaximum( 99999 );

    connect( m_pSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onFrameCountChange(int) ) );
    connect( m_pEnableCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onEnableClicked(bool) ) );
    connect( m_pShowCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onShowImageClicked(bool) ) );
    connect( m_pPushButton, SIGNAL( clicked(bool) ), this, SLOT( onComputeClicked() ) );
    connect( m_pUploadButton, SIGNAL( clicked() ), this, SLOT( onUploadClicked() ) );
}


ShadingControl::~ShadingControl()
{
}


void ShadingControl::setCamera ( Dc1394Camera*      pCamera,
                                 AcquisitionThread* pThread )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        m_bRefreshRequired = false;
        return;
    }

    /* check feature support */
    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->Shading )
    {
        /* Shading is not supported */
        setEnabled(false);
        return;
    }

    setEnabled(true);
    m_pUploadButton->setEnabled( !pThread->isRunning() );
    pollStateFromCamera();
}


void ShadingControl::pollStateFromCamera()
{ 
    dc1394error_t eRet;
    uint32_t      nFrameCount;
    dc1394bool_t  bActive;
    dc1394bool_t  bComputing;
    dc1394bool_t  bShowImage;

    eRet = dc1394_avt_get_shading( m_pCamObject->psCamInternal(),
                                   &bActive,
                                   &bComputing,
                                   &bShowImage,
                                   &nFrameCount );
    LOGFCALL_LIBDC ( "dc1394_avt_get_shading", eRet );

    m_nFrameCount = (int) nFrameCount;
    m_bActive  = ( DC1394_TRUE == bActive );
    m_bShowImage = ( DC1394_TRUE == bShowImage );
    m_bRefreshRequired = ( DC1394_TRUE == bComputing );

    m_pEnableCheckBox->setChecked( m_bActive );
    m_pShowCheckBox->setChecked( m_bShowImage );
    m_pSpinBox->setValue( m_nFrameCount );

    enableControls();
}


void ShadingControl::enableControls()
{
    m_pShowCheckBox->setEnabled( m_bActive && !m_bRefreshRequired );
}


void ShadingControl::onStateChangedByTwinControl ( void )
{
    ShadingControl* pSender = qobject_cast<ShadingControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nFrameCount = pSender->m_nFrameCount;
    m_bActive = pSender->m_bActive;
    m_bShowImage = pSender->m_bShowImage;
    m_bRefreshRequired = pSender->m_bRefreshRequired;

    m_pEnableCheckBox->setChecked( m_bActive );
    m_pShowCheckBox->setChecked( m_bShowImage );
    m_pSpinBox->setValue( m_nFrameCount );

    enableControls();
}


void ShadingControl::setAcquisitionState( bool bActive )
{
    m_pUploadButton->setEnabled( !bActive );
}

void ShadingControl::onFrameCountChange ( int nCount )
{
    if( m_nFrameCount != nCount )
    {
        m_nFrameCount = nCount;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_shading( m_pCamObject->psCamInternal(),
                                       m_bActive? DC1394_TRUE : DC1394_FALSE,
                                       DC1394_FALSE,
                                       m_bShowImage? DC1394_TRUE : DC1394_FALSE,
                                       (uint32_t) m_nFrameCount );
        LOGFCALL_LIBDC ( "dc1394_avt_set_shading", eRet );

        emit featureStateChanged();
        emit ShadingChange( m_bActive, m_bShowImage );
    }
}


void ShadingControl::onEnableClicked ( bool bChecked )
{
    if( m_bActive != bChecked )
    {
        m_bActive = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_shading( m_pCamObject->psCamInternal(),
                                       m_bActive? DC1394_TRUE : DC1394_FALSE,
                                       DC1394_FALSE,
                                       m_bShowImage? DC1394_TRUE : DC1394_FALSE,
                                       (uint32_t) m_nFrameCount );
        LOGFCALL_LIBDC ( "dc1394_avt_set_shading", eRet );

        enableControls();

        emit featureStateChanged();
        emit ShadingChange( m_bActive, m_bShowImage );
    }
}


void ShadingControl::onShowImageClicked ( bool bChecked )
{
    if( m_bShowImage != bChecked )
    {
        m_bShowImage = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_shading( m_pCamObject->psCamInternal(),
                                       m_bActive? DC1394_TRUE : DC1394_FALSE,
                                       DC1394_FALSE,
                                       m_bShowImage? DC1394_TRUE : DC1394_FALSE,
                                       (uint32_t) m_nFrameCount );
        LOGFCALL_LIBDC ( "dc1394_avt_set_shading", eRet );

        emit featureStateChanged();
        emit ShadingChange( m_bActive, m_bShowImage );
    }
}


void ShadingControl::onComputeClicked()
{
    if( !m_bRefreshRequired )
    {
        setEnabled( false );

        dc1394error_t eRet;
        eRet = dc1394_avt_set_shading( m_pCamObject->psCamInternal(),
                                       m_bActive? DC1394_TRUE : DC1394_FALSE,
                                       DC1394_TRUE,
                                       m_bShowImage? DC1394_TRUE : DC1394_FALSE,
                                       (uint32_t) m_nFrameCount );
        LOGFCALL_LIBDC ( "dc1394_avt_set_shading", eRet );

        setEnabled( true );
    }
}


void ShadingControl::onUploadClicked()
{
    QString strFileName = QFileDialog::getOpenFileName( this, "Open Shading Image", QDir::homePath (), tr("Shading Files (*.sci);;All Files (*.*)"));
    Logger::log("Trying to open file %s for shading upload", strFileName.toAscii().data() );

    dc1394error_t eRet;
    uint32_t      nMaxImageSize;

    eRet = dc1394_avt_get_shading_info(m_pCamObject->psCamInternal(), &nMaxImageSize);
    LOGFCALL_LIBDC ( "dc1394_avt_get_shading_info", eRet );

    if( eRet != DC1394_SUCCESS )
    {
        return;
    }

    QFile shadingFile( strFileName );
    if( shadingFile.size() != (qint64) nMaxImageSize )
    {
        QMessageBox::critical( this, tr("Error"), tr("File size does not match camera shading image size! - Shading images need to match the camera's max image size") );
        return;
    }

    if( ! shadingFile.open(QIODevice::ReadOnly) )
    {
        Logger::log( "Opening file %s failed", strFileName.toAscii().data() );
        QMessageBox::critical( this, tr("Error"), tr("Could not open File for read access!") );
        return;
    }
    shadingFile.seek(0);

    QByteArray data = shadingFile.read( shadingFile.size() );
    shadingFile.close();

    if(data.size()!=shadingFile.size())
    {
        Logger::log("Could not read file");
        QMessageBox::critical( this, tr("Error"), tr("Could not read file!") );
        return;
    }

    eRet = dc1394_avt_write_shading_img( m_pCamObject->psCamInternal(), (unsigned char*) data.data(), data.size() );
    LOGFCALL_LIBDC ( "dc1394_avt_write_shading_img", eRet );
}
