/** \file shuttercontrol.cpp
 *  \brief This file contains the implementation of the feature control class ShutterControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "shuttercontrol.h"

#include "extdshuttercontrol.h"

ShutterControl::ShutterControl( QWidget* pParent )
 : StandardFeatureControl( pParent, DC1394_FEATURE_SHUTTER, "Shutter" )
{
    /* duplicate featureStateChanged signal for extd. shutter controls */
    connect( this, SIGNAL( featureStateChanged(void) ), this, SIGNAL( ShutterChanged() ) );
    connect( this, SIGNAL( featureStateChanged(void) ), this, SLOT( SimpleShutterConfigured() ) );
}


void ShutterControl::setCamera( Dc1394Camera*      pCamera, 
                                AcquisitionThread* pThread )
{
    StandardFeatureControl::setCamera( pCamera, pThread );

    if( NULL == pCamera )
    {
        return;
    }

    /* read autoshutter range and timebase - standard shutter value is not relevant when
       auto shutter is active and auto shutter range exceeds the valid standard shutter range. */
    dc1394error_t eRet;
    uint32_t nMin, nMax;
    eRet = dc1394_avt_get_auto_shutter( m_pCamObject->psCamInternal(), &nMin, &nMax );
    LOGFCALL_LIBDC ( "dc1394_avt_get_auto_shutter", eRet );

    uint32_t nTimeBase;
    eRet = dc1394_avt_get_timebase( m_pCamObject->psCamInternal(), &nTimeBase );
    LOGFCALL_LIBDC ( "dc1394_avt_get_timebase", eRet );

    if(DC1394_SUCCESS==eRet)
    {
        m_nCurrentTimeBaseIndex = nTimeBase;
    }
    else
    {
        m_nCurrentTimeBaseIndex = 1;
    }
    m_nCurrentMaxAutoShutter = nMax;

    checkAutoShutterRange();
    adjustSpinBox( !m_bRefreshRequired || !m_bAutoShutterBeyondRange );
}


void ShutterControl::onStateChangedByTwinControl(void)
{
    StandardFeatureControl::onStateChangedByTwinControl();
    adjustSpinBox( !m_bRefreshRequired || !m_bAutoShutterBeyondRange );
}


void ShutterControl::onExtendedShutterValueAdjusted_autoconnect(void)
{
    adjustSpinBox( false );
}


void ShutterControl::onExtendedShutterModeChanged_autoconnect(void)
{
    ExtdShutterControl* pSender = qobject_cast<ExtdShutterControl*> ( sender() );

    bool bStateChanged = false;
    if( m_bAutoActive != pSender->m_bAutoActive )
    {
        m_bAutoActive = pSender->m_bAutoActive;
        m_pAutoCheckBox->setChecked( m_bAutoActive );
        bStateChanged = true;
    }

    if( m_bRefreshRequired != pSender->m_bRefreshRequired )
    {
        m_bRefreshRequired = pSender->m_bRefreshRequired;
        bStateChanged = true;
    }

    adjustSpinBox( !m_bRefreshRequired || !m_bAutoShutterBeyondRange );

    if( bStateChanged )
    {
        enableControls();
    }
}


void ShutterControl::onAutoShutterChange_autoconnect( int /*nMin*/, int nMax )
{
    m_nCurrentMaxAutoShutter = nMax;
    checkAutoShutterRange();
    adjustSpinBox( !m_bRefreshRequired || !m_bAutoShutterBeyondRange );
}


void ShutterControl::onTimeBaseChange_autoconnect( int nTimeBase )
{
    m_nCurrentTimeBaseIndex = nTimeBase;
    checkAutoShutterRange();
    adjustSpinBox( !m_bRefreshRequired || !m_bAutoShutterBeyondRange );
}


void ShutterControl::SimpleShutterConfigured()
{
    adjustSpinBox( !m_bRefreshRequired || !m_bAutoShutterBeyondRange );
}


void ShutterControl::checkAutoShutterRange()
{
    int nTimeBaseVal;
    switch( m_nCurrentTimeBaseIndex )
    {
        case 0:
            nTimeBaseVal = 1;
        case 1:
            nTimeBaseVal = 2;
        case 2:
            nTimeBaseVal = 5;
        case 3:
            nTimeBaseVal = 10;
        case 4:
            nTimeBaseVal = 20;
        case 5:
            nTimeBaseVal = 50;
        case 6:
            nTimeBaseVal = 100;
        case 7:
            nTimeBaseVal = 200;
        case 8:
            nTimeBaseVal = 500;
        default:
            nTimeBaseVal = 1000;
    }
    m_bAutoShutterBeyondRange = ( nTimeBaseVal * m_pSlider->maximum() < m_nCurrentMaxAutoShutter );
}

void ShutterControl::adjustSpinBox( bool bRelevant )
{
    QPalette p = palette();
    if( !bRelevant )
    {
        p.setColor(QPalette::Text, QColor(140,140,140));
        m_pSpinBox->setPrefix("[");
        m_pSpinBox->setSuffix("]");
    }
    else
    {
        m_pSpinBox->setPrefix("");
        m_pSpinBox->setSuffix("");
    }
    m_pSpinBox->setPalette(p);
}
