/** \file timebasecontrol.cpp
 *  \brief This file contains the implementation of the feature control class TimeBaseControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "timebasecontrol.h"

#include <dc1394/dc1394.h>
#include <dc1394/vendor/avt.h>

TimeBaseControl::TimeBaseControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_TimeBaseControl()
{
    setupUi ( this );

    m_pComboBox->clear();
    m_pComboBox->addItem( QString::fromLocal8Bit("1 µs"), QVariant( 0 ) );
    m_pComboBox->addItem( QString::fromLocal8Bit("2 μs"), QVariant( 1 ) );
    m_pComboBox->addItem( QString::fromLocal8Bit("5 µs"), QVariant( 2 ) );
    m_pComboBox->addItem( QString::fromLocal8Bit("10 µs"), QVariant( 3 ) );
    m_pComboBox->addItem( QString::fromLocal8Bit("20 µs"), QVariant( 4 ) );
    m_pComboBox->addItem( QString::fromLocal8Bit("50 µs"), QVariant( 5 ) );
    m_pComboBox->addItem( QString::fromLocal8Bit("100 µs"), QVariant( 6 ) );
    m_pComboBox->addItem( QString::fromLocal8Bit("200 µs"), QVariant( 7 ) );
    m_pComboBox->addItem( QString::fromLocal8Bit("500 µs"), QVariant( 8 ) );
    m_pComboBox->addItem( QString::fromLocal8Bit("1000 µs"), QVariant( 9 ) );

    connect( m_pComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onChange(int) ) );
}


TimeBaseControl::~TimeBaseControl()
{
}


void TimeBaseControl::setCamera( Dc1394Camera*      pCamera, 
                                 AcquisitionThread* pThread )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    /* check feature support */
    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->TimeBase )
    {
        /* time base is not supported */
        setEnabled(false);
        return;
    }

    setEnabled(true);
    m_pComboBox->setEnabled( !pThread->isRunning() );
    pollStateFromCamera();
}


void TimeBaseControl::pollStateFromCamera()
{
    dc1394error_t eRet;
    uint32_t nVal;
    eRet = dc1394_avt_get_timebase( m_pCamObject->psCamInternal(), &nVal );
    LOGFCALL_LIBDC ( "dc1394_avt_get_timebase", eRet );

    m_nCurrentTimeBase = (int) nVal;
    m_pComboBox->setCurrentIndex( m_pComboBox->findData( QVariant( m_nCurrentTimeBase ) ) );
}


void TimeBaseControl::onStateChangedByTwinControl(void)
{
    TimeBaseControl* pSender = qobject_cast<TimeBaseControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nCurrentTimeBase = pSender->m_nCurrentTimeBase;
    m_pComboBox->setCurrentIndex( m_pComboBox->findData( QVariant( m_nCurrentTimeBase ) ) );
}

void TimeBaseControl::setAcquisitionState( bool bActive )
{
    m_pComboBox->setEnabled( !bActive );
}


void TimeBaseControl::onChange( int nIndex )
{
    if( m_pComboBox->itemData( nIndex ).toInt() != m_nCurrentTimeBase )
    {
        m_nCurrentTimeBase = m_pComboBox->itemData( nIndex ).toInt();

        dc1394error_t eRet;
        eRet = dc1394_avt_set_timebase( m_pCamObject->psCamInternal(), 
                                        (uint32_t) m_nCurrentTimeBase );
        LOGFCALL_LIBDC ( "dc1394_avt_set_timebase", eRet );

        emit featureStateChanged();
        emit TimeBaseChange( m_nCurrentTimeBase );
    }
}
