/** \file triggercontrol.cpp
 *  \brief This file contains the implementation of the feature control class TriggerControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "triggercontrol.h"

#include <dc1394/dc1394.h>

TriggerControl::TriggerControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_TriggerControl()
{
    setupUi ( this );
    m_pPolComboBox->clear();
    m_pPolComboBox->addItem( "low active", QVariant( (int) DC1394_TRIGGER_ACTIVE_LOW ) );
    m_pPolComboBox->addItem( "high active", QVariant( (int) DC1394_TRIGGER_ACTIVE_HIGH ) );

    connect( m_pEnableCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onEnableClicked(bool) ) );
    connect( m_pPolComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onPolarityChange(int) ) );
    connect( m_pModeComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onModeChange(int) ) );
    connect( m_pSourceComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onSourceChange(int) ) );
    
}


TriggerControl::~TriggerControl()
{
}


void TriggerControl::setCamera( Dc1394Camera*      pCamera, 
                                AcquisitionThread* pThread )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    /* check feature support */
    const dc1394feature_info_t* psFeatureInfo = pCamera->getFeatureInfo( DC1394_FEATURE_TRIGGER );
    if ( DC1394_FALSE == psFeatureInfo->available )
    {
        /* feature is not supported */
        setEnabled(false);
        return;
    }

    /* check feature capabilities */
    m_bPolaritySupported = ( DC1394_TRUE == psFeatureInfo->polarity_capable );
    m_pPolComboBox->setEnabled( m_bPolaritySupported );

    m_pModeComboBox->clear();
    QVector<cc1394_triggermodeitem> vecModeList;
    m_pCamObject->getTriggerModeList( vecModeList );
    foreach( cc1394_triggermodeitem item, vecModeList )
    {
        m_pModeComboBox->addItem( item.m_strName, QVariant( (int) item.m_eMode ) );
    }

    m_bSWTriggerSupported = false;
    m_pSourceComboBox->clear();
    QVector<cc1394_triggersourceitem> vecSourceList;
    m_pCamObject->getTriggerSourceList( vecSourceList );
    foreach( cc1394_triggersourceitem item, vecSourceList )
    {
        m_pSourceComboBox->addItem( item.m_strName, QVariant( (int) item.m_eSource ) );
        if( DC1394_TRIGGER_SOURCE_SOFTWARE == item.m_eSource )
        {
            m_bSWTriggerSupported = true;
        }
    }

    m_pEnableCheckBox->setEnabled( !pThread->isRunning() );
    m_pModeComboBox->setEnabled( !pThread->isRunning() );
    m_pPolComboBox->setEnabled( !pThread->isRunning() );
    m_pSourceComboBox->setEnabled( !pThread->isRunning() );
    m_pSWTriggerCheckBox->setEnabled( !pThread->isRunning() && m_bSWTriggerSupported );

    setEnabled(true);
    pollStateFromCamera();
}


void TriggerControl::setAcquisitionState( bool bActive )
{
    m_pEnableCheckBox->setEnabled( !bActive );
    m_pModeComboBox->setEnabled( !bActive );
    m_pPolComboBox->setEnabled( !bActive );
    m_pSourceComboBox->setEnabled( !bActive );
    m_pSWTriggerCheckBox->setEnabled( !bActive && m_bSWTriggerSupported );
}


void TriggerControl::pollStateFromCamera()
{
    dc1394error_t eRet;

    if( m_bPolaritySupported )
    {
        eRet = dc1394_external_trigger_get_polarity( m_pCamObject->psCamInternal(), &m_ePolarity );
        LOGFCALL_LIBDC ( "dc1394_external_trigger_get_polarity", eRet );
        m_pPolComboBox->setCurrentIndex( m_pPolComboBox->findData( QVariant( m_ePolarity ) ) );
    }

    dc1394switch_t ePower;
    eRet = dc1394_external_trigger_get_power( m_pCamObject->psCamInternal(), &ePower);
    LOGFCALL_LIBDC ( "dc1394_external_trigger_get_power", eRet );
    m_bActive = ( DC1394_ON == ePower );
    m_pEnableCheckBox->setChecked( m_bActive );

    eRet = dc1394_external_trigger_get_mode( m_pCamObject->psCamInternal(), &m_eMode );
    LOGFCALL_LIBDC ( "dc1394_external_trigger_get_mode", eRet );
    m_pModeComboBox->setCurrentIndex( m_pModeComboBox->findData( QVariant( m_eMode ) ) );

    eRet = dc1394_external_trigger_get_source( m_pCamObject->psCamInternal(), &m_eSource );
    LOGFCALL_LIBDC ( "dc1394_external_trigger_get_source", eRet );
    m_pSourceComboBox->setCurrentIndex( m_pSourceComboBox->findData( QVariant( m_eSource ) ) );

    eRet = dc1394_software_trigger_get_power( m_pCamObject->psCamInternal(), &ePower);
    LOGFCALL_LIBDC ( "dc1394_software_trigger_get_power", eRet );
    m_bSwTriggerActive = ( DC1394_ON == ePower );
    m_pSWTriggerCheckBox->setChecked( m_bSwTriggerActive );

    m_bRefreshRequired = m_bSwTriggerActive;
}


void TriggerControl::onStateChangedByTwinControl(void)
{
    TriggerControl* pSender = qobject_cast<TriggerControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_bActive = pSender->m_bActive;
    m_bSwTriggerActive = pSender->m_bSwTriggerActive;
    m_ePolarity = pSender->m_ePolarity;
    m_eMode = pSender->m_eMode;
    m_eSource = pSender->m_eSource;

    m_pEnableCheckBox->setChecked( m_bActive );
    m_pModeComboBox->setCurrentIndex( m_pModeComboBox->findData( QVariant( m_eMode ) ) );
    m_pSourceComboBox->setCurrentIndex( m_pSourceComboBox->findData( QVariant( m_eSource ) ) );
    m_pSWTriggerCheckBox->setChecked( m_bSwTriggerActive );

    m_bRefreshRequired = m_bSwTriggerActive;
}


void TriggerControl::onEnableClicked( bool bChecked )
{
    if( m_bActive != bChecked )
    {
        m_bActive = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_external_trigger_set_power( m_pCamObject->psCamInternal(),
                                                  m_bActive? DC1394_ON : DC1394_OFF );
        LOGFCALL_LIBDC ( "dc1394_external_trigger_set_power", eRet );
    }
}


void TriggerControl::onPolarityChange( int nIndex )
{
    if( m_ePolarity != (dc1394trigger_polarity_t) m_pPolComboBox->itemData( nIndex ).toInt() )
    {
        m_ePolarity = (dc1394trigger_polarity_t) m_pPolComboBox->itemData( nIndex ).toInt();

        dc1394error_t eRet;
        eRet = dc1394_external_trigger_set_polarity( m_pCamObject->psCamInternal(), m_ePolarity );
        LOGFCALL_LIBDC ( "dc1394_external_trigger_set_polarity", eRet );
    }
}


void TriggerControl::onModeChange( int nIndex )
{
    if( m_eMode != (dc1394trigger_mode_t) m_pModeComboBox->itemData( nIndex ).toInt() )
    {
        m_eMode = (dc1394trigger_mode_t) m_pModeComboBox->itemData( nIndex ).toInt();

        dc1394error_t eRet;
        eRet = dc1394_external_trigger_set_mode( m_pCamObject->psCamInternal(), m_eMode );
        LOGFCALL_LIBDC ( "dc1394_external_trigger_set_mode", eRet );
    }
}


void TriggerControl::onSourceChange( int nIndex )
{
    if( m_eSource != (dc1394trigger_source_t) m_pSourceComboBox->itemData( nIndex ).toInt() )
    {
        m_eSource = (dc1394trigger_source_t) m_pSourceComboBox->itemData( nIndex ).toInt();

        dc1394error_t eRet;
        eRet = dc1394_external_trigger_set_source( m_pCamObject->psCamInternal(), m_eSource);
        LOGFCALL_LIBDC ( "dc1394_external_trigger_set_source", eRet );
    }
}


void TriggerControl::onSwTriggerClicked( bool bChecked )
{
    if( m_bSwTriggerActive != bChecked )
    {
        m_bSwTriggerActive = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_software_trigger_set_power( m_pCamObject->psCamInternal(),
                                                  m_bSwTriggerActive? DC1394_ON : DC1394_OFF );
        m_bRefreshRequired = m_bSwTriggerActive;
    }
}
