/** \file whitebalancecontrol.cpp
 *  \brief This file contains the implementation of the feature control class WhitebalanceControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "whitebalancecontrol.h"

WhitebalanceControl::WhitebalanceControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_WhitebalanceControl()
{
    setupUi ( this );

    // show the features name to the user
    m_pNameLabel->setText( "Whitebalance" );

    // connect controls
    connect( m_pSpinBox_UB, SIGNAL( valueChanged(int) ), this, SLOT( onValueChange_UB(int) ) );
    connect( m_pSpinBox_VR, SIGNAL( valueChanged(int) ), this, SLOT( onValueChange_VR(int) ) );
    connect( m_pEnableCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onEnableClicked(bool) ) );
    connect( m_pAutoCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onAutoClicked(bool) ) );
    connect( m_pOnePushButton, SIGNAL( clicked() ), this, SLOT( onOnePushClicked() ) );
}


WhitebalanceControl::~WhitebalanceControl()
{
}


void WhitebalanceControl::setCamera( Dc1394Camera*      pCamera, 
                                     AcquisitionThread* /*pThread*/ )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    const dc1394feature_info_t* psFeatureInfo = pCamera->getFeatureInfo( DC1394_FEATURE_WHITE_BALANCE );
    if ( DC1394_FALSE == psFeatureInfo->available )
    {
        /* feature is not supported */
        setEnabled(false);
        return;
    }

    m_pNameLabel->setEnabled(true);
    m_pSlider_UB->setEnabled(true);
    m_pSlider_VR->setEnabled(true);
    m_pSpinBox_UB->setEnabled(true);
    m_pSpinBox_VR->setEnabled(true);

    /* read min/max */
    m_pSlider_UB->setMinimum( (int) psFeatureInfo->min );
    m_pSlider_VR->setMinimum( (int) psFeatureInfo->min );
    m_pSlider_UB->setMaximum( (int) psFeatureInfo->max );
    m_pSlider_VR->setMaximum( (int) psFeatureInfo->max );
    m_pSpinBox_UB->setMinimum( (int) psFeatureInfo->min );
    m_pSpinBox_VR->setMinimum( (int) psFeatureInfo->min );
    m_pSpinBox_UB->setMaximum( (int) psFeatureInfo->max );
    m_pSpinBox_VR->setMaximum( (int) psFeatureInfo->max );

    /* check On/Off Capablility */
    m_pEnableCheckBox->setEnabled( DC1394_FALSE != psFeatureInfo->on_off_capable );
    if ( DC1394_FALSE == psFeatureInfo->on_off_capable )
    {
        /* feature is non-switchable, so its considered to be permanently active */
        m_pEnableCheckBox->setChecked(true);
        m_bPowerState = true;
    }

    /* check auto capabilities ('feature modes') */
    m_bAutoSupported = m_pCamObject->featureModeSupported( DC1394_FEATURE_WHITE_BALANCE,
                                                           DC1394_FEATURE_MODE_AUTO );
    m_bOnePushAutoSupported = m_pCamObject->featureModeSupported( DC1394_FEATURE_WHITE_BALANCE,
                                                                   DC1394_FEATURE_MODE_ONE_PUSH_AUTO );

    /* read feature state from info struct - data is only current when the camera object
       has been just created. */
    m_nValue_UB = (int) psFeatureInfo->BU_value;
    m_nValue_VR = (int) psFeatureInfo->RV_value;
    m_pSpinBox_UB->setValue( m_nValue_UB );
    m_pSpinBox_VR->setValue( m_nValue_VR );

    if ( m_pEnableCheckBox->isEnabled() )
    {
        m_bPowerState = ( DC1394_ON == psFeatureInfo->is_on );
        m_pEnableCheckBox->setChecked( m_bPowerState );
    }

    if ( m_bAutoSupported )
    {
        m_bAutoActive = ( DC1394_FEATURE_MODE_AUTO == psFeatureInfo->current_mode );
    }
    else
    {
        m_bAutoActive = false;
    }
    m_pAutoCheckBox->setChecked( m_bAutoActive );

    m_bRefreshRequired = ( DC1394_FEATURE_MODE_AUTO == psFeatureInfo->current_mode ||
                           DC1394_FEATURE_MODE_ONE_PUSH_AUTO == psFeatureInfo->current_mode );
    enableControls();
    setEnabled(true);
}


void WhitebalanceControl::pollStateFromCamera()
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    bool bStateChanged = false;

    /* get current value */
    uint32_t nVal_UB, nVal_VR;
    dc1394error_t eRet  = dc1394_feature_whitebalance_get_value( m_pCamObject->psCamInternal(), &nVal_UB, &nVal_VR);
    LOGFCALL_LIBDC ( "dc1394_feature_whitebalance_get_value", eRet );

    if( m_nValue_UB != (int) nVal_UB )
    {
        m_nValue_UB = (int) nVal_UB;
        m_pSpinBox_UB->setValue( m_nValue_UB );
        bStateChanged = true;
    }
    if( m_nValue_VR != (int) nVal_VR )
    {
        m_nValue_VR = (int) nVal_VR;
        m_pSpinBox_VR->setValue( m_nValue_VR );
        bStateChanged = true;
    }

    /* get current on/off state */
    dc1394switch_t eSwitch;
    if ( m_pEnableCheckBox->isEnabled() )
    {
        eRet = dc1394_feature_get_power( m_pCamObject->psCamInternal(), DC1394_FEATURE_WHITE_BALANCE, &eSwitch);
        LOGFCALL_LIBDC ( "dc1394_feature_get_power", eRet );

        if( m_bPowerState != ( DC1394_ON == eSwitch ) )
        {
            m_bPowerState = ( DC1394_ON == eSwitch );
            m_pEnableCheckBox->setChecked( m_bPowerState );
            bStateChanged = true;
        }
    }

    /* get current auto state */
    dc1394feature_mode_t eMode;
    eRet = dc1394_feature_get_mode( m_pCamObject->psCamInternal(), DC1394_FEATURE_WHITE_BALANCE, &eMode);
    LOGFCALL_LIBDC ( "dc1394_feature_get_mode", eRet );
    bool bRefreshRequiredNew = ( DC1394_FEATURE_MODE_AUTO == eMode ||
                           DC1394_FEATURE_MODE_ONE_PUSH_AUTO == eMode );
    if( m_bRefreshRequired != bRefreshRequiredNew )
    {
        m_bRefreshRequired = bRefreshRequiredNew;
        bStateChanged = true;
    }

    if ( m_pAutoCheckBox->isEnabled() && m_bAutoActive != ( DC1394_FEATURE_MODE_AUTO == eMode ) )
    {
        m_bAutoActive = ( DC1394_FEATURE_MODE_AUTO == eMode );
        m_pAutoCheckBox->setChecked( m_bAutoActive );
        bStateChanged = true;
    }

    if( bStateChanged )
    {
        enableControls();
    }
}


void WhitebalanceControl::onStateChangedByTwinControl(void)
{
    WhitebalanceControl* pSender = qobject_cast<WhitebalanceControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nValue_UB = pSender->m_nValue_UB;
    m_nValue_VR = pSender->m_nValue_VR;
    m_bAutoActive = pSender->m_bAutoActive;
    m_bRefreshRequired = pSender->m_bRefreshRequired;
    m_pSpinBox_UB->setValue( m_nValue_UB );
    m_pSpinBox_VR->setValue( m_nValue_VR );
    m_pAutoCheckBox->setChecked( m_bAutoActive );

    enableControls();
}


void WhitebalanceControl::onValueChange_UB(int nValue)
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nValue_UB != nValue )
    {
        m_nValue_UB = nValue;

        /* configure camera */
        dc1394error_t eRet  = dc1394_feature_whitebalance_set_value( m_pCamObject->psCamInternal(),
                                                                     (uint32_t) m_nValue_UB,
                                                                     (uint32_t) m_nValue_VR);
        LOGFCALL_LIBDC ( "dc1394_feature_whitebalance_set_value", eRet );
    }
    emit featureStateChanged();
}


void WhitebalanceControl::onValueChange_VR(int nValue)
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nValue_VR != nValue )
    {
        m_nValue_VR = nValue;

        /* configure camera */
        dc1394error_t eRet  = dc1394_feature_whitebalance_set_value( m_pCamObject->psCamInternal(),
                                                                     (uint32_t) m_nValue_UB,
                                                                     (uint32_t) m_nValue_VR);
        LOGFCALL_LIBDC ( "dc1394_feature_whitebalance_set_value", eRet );

        emit featureStateChanged();
    }
}


void WhitebalanceControl::onEnableClicked(bool bChecked)
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_bPowerState != bChecked )
    {
        m_bPowerState = bChecked;

        /* configure camera */
        dc1394error_t eRet  = dc1394_feature_set_power( m_pCamObject->psCamInternal(),
                                                        DC1394_FEATURE_WHITE_BALANCE,
                                                        m_bPowerState? DC1394_ON : DC1394_OFF );
        LOGFCALL_LIBDC ( "dc1394_feature_set_power", eRet );

        emit featureStateChanged();
    }
}


void WhitebalanceControl::onAutoClicked(bool bChecked)
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_bAutoActive != bChecked )
    {
        m_bAutoActive = bChecked;

        /* configure camera */
        dc1394error_t eRet  = dc1394_feature_set_mode( m_pCamObject->psCamInternal(),
                                                       DC1394_FEATURE_WHITE_BALANCE,
                                                       m_bAutoActive? DC1394_FEATURE_MODE_AUTO :
                                                                      DC1394_FEATURE_MODE_MANUAL );
        LOGFCALL_LIBDC ( "dc1394_feature_set_mode", eRet );

        m_bRefreshRequired = m_bAutoActive;
        enableControls();
        emit featureStateChanged();
    }
}


void WhitebalanceControl::onOnePushClicked()
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    dc1394error_t eRet  = dc1394_feature_set_mode( m_pCamObject->psCamInternal(),
                                                   DC1394_FEATURE_WHITE_BALANCE,
                                                   DC1394_FEATURE_MODE_ONE_PUSH_AUTO );
    LOGFCALL_LIBDC ( "dc1394_feature_set_mode", eRet );

    m_bRefreshRequired = true;
    enableControls();
    emit featureStateChanged();
}


void WhitebalanceControl::enableControls()
{
    bool bControlsEnabled = !m_bRefreshRequired;
    if( !bControlsEnabled && m_pSlider_UB->hasFocus() )
    {
        m_pSlider_UB->clearFocus();
    }
    m_pSlider_UB->setEnabled( bControlsEnabled );
    if( !bControlsEnabled && m_pSlider_VR->hasFocus() )
    {
        m_pSlider_VR->clearFocus();
    }
    m_pSlider_VR->setEnabled( bControlsEnabled );

    if( !bControlsEnabled && m_pSpinBox_UB->hasFocus() )
    {
        m_pSpinBox_UB->clearFocus();
    }
    m_pSpinBox_UB->setEnabled( bControlsEnabled );
    if( !bControlsEnabled && m_pSpinBox_VR->hasFocus() )
    {
        m_pSpinBox_VR->clearFocus();
    }
    m_pSpinBox_VR->setEnabled( bControlsEnabled );

    if( !bControlsEnabled && m_pOnePushButton->hasFocus() )
    {
        m_pOnePushButton->clearFocus();
    }
    m_pOnePushButton->setEnabled( bControlsEnabled && m_bOnePushAutoSupported );

    bool bAutoCheckBoxEnabled = !m_bRefreshRequired || m_bAutoActive;
    if( !bAutoCheckBoxEnabled && m_pAutoCheckBox->hasFocus() )
    {
        m_pAutoCheckBox->clearFocus();
    }
    m_pAutoCheckBox->setEnabled( bAutoCheckBoxEnabled && m_bAutoSupported );
}
