/** \file main.cpp
 *  \brief This file contains the applications main() routine
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <QApplication>
#include <QFileInfo>
#include <QFile>
#include <QDir>
#include <QSplashScreen>
#include <QTime>
#include <QMessageBox>

#include "mainwindow.h"

/**
 * helper routine for the creation of the default configuration file
 */
void checkProfile()
{
    /* check/create profile dir */
    QFileInfo dirinfo ( QDir::home().absolutePath()+"/.cc1394" );
    if ( ! dirinfo.isDir() )
    {
        QDir homedir = QDir::home();
        homedir.mkdir ( ".cc1394" );
    }

    /* check/create file featurecontrollayout.xml */
    QFileInfo xmlinfo ( QDir::home().absolutePath()+"/.cc1394/featurecontrollayout.xml" );
    if ( ! xmlinfo.exists() )
    {
        QFile xmlfile ( xmlinfo.absoluteFilePath() );
        xmlfile.open ( QIODevice::WriteOnly );
        xmlfile.write ( "<?xml version='1.0' encoding='UTF-8'?>\n" );
        xmlfile.write ( "<featurecontrollayout version=\"0\">\n" );
        xmlfile.write ( "  <featurecontrolpage title=\"Acq\">\n" );
        xmlfile.write ( "    <feature name=\"acquisition\"/>\n" );
        xmlfile.write ( "    <feature name=\"multishotcount\"/>\n" );
        xmlfile.write ( "    <feature name=\"isospeed\"/>\n" );
        xmlfile.write ( "    <feature name=\"videomode\"/>\n" );
        xmlfile.write ( "    <feature name=\"colorcoding\"/>\n" );
        xmlfile.write ( "    <feature name=\"framerate\"/>\n" );
        xmlfile.write ( "    <feature name=\"packetsize\"/>\n" );
        xmlfile.write ( "    <feature name=\"aoi\"/>\n" );
        xmlfile.write ( "  </featurecontrolpage>\n" );
        xmlfile.write ( "  <featurecontrolpage title=\"Exp\">\n" );
        xmlfile.write ( "    <feature name=\"brightness\"/>\n" );
        xmlfile.write ( "    <feature name=\"exposure\"/>\n" );
        xmlfile.write ( "    <feature name=\"shutter\"/>\n" );
        xmlfile.write ( "    <feature name=\"extdshutter\"/>\n" );
        xmlfile.write ( "    <feature name=\"gain\"/>\n" );
        xmlfile.write ( "    <feature name=\"gamma\"/>\n" );
        xmlfile.write ( "  </featurecontrolpage>\n" );
        xmlfile.write ( "  <featurecontrolpage title=\"Col\">\n" );
        xmlfile.write ( "    <feature name=\"colorcorrection\"/>\n" );
        xmlfile.write ( "    <feature name=\"hue\"/>\n" );
        xmlfile.write ( "    <feature name=\"whitebalance\"/>\n" );
        xmlfile.write ( "    <feature name=\"saturation\"/>\n" );
        xmlfile.write ( "  </featurecontrolpage>\n" );
        xmlfile.write ( "  <featurecontrolpage title=\"Adj\">\n" );
        xmlfile.write ( "    <feature name=\"sharpness\"/>\n" );
        xmlfile.write ( "    <feature name=\"lut\"/>\n" );
        xmlfile.write ( "    <feature name=\"shading\"/>\n" );
        xmlfile.write ( "    <feature name=\"blemish\"/>\n" );
        xmlfile.write ( "    <feature name=\"dsnu\"/>\n" );
        xmlfile.write ( "    <feature name=\"defectpixel\"/>\n" );
        xmlfile.write ( "    <feature name=\"channeladjustgain\"/>\n" );
        xmlfile.write ( "    <feature name=\"channeladjustoffset\"/>\n" );
        xmlfile.write ( "    <feature name=\"lowsmear\"/>\n" );
        xmlfile.write ( "  </featurecontrolpage>\n" );
        xmlfile.write ( "  <featurecontrolpage title=\"Adv 1\">\n" );
        xmlfile.write ( "    <feature name=\"mirror\"/>\n" );
        xmlfile.write ( "    <feature name=\"globalresrelshutter\"/>\n" );
        xmlfile.write ( "    <feature name=\"led\"/>\n" );
        xmlfile.write ( "    <feature name=\"lownoisebinning\"/>\n" );
        xmlfile.write ( "    <feature name=\"timebase\"/>\n" );
        xmlfile.write ( "    <feature name=\"testimage\"/>\n" );
        xmlfile.write ( "    <feature name=\"hsnr\"/>\n" );
        xmlfile.write ( "    <feature name=\"hdr\"/>\n" );
        xmlfile.write ( "  </featurecontrolpage>\n" );
        xmlfile.write ( "  <featurecontrolpage title=\"Adv 2\">\n" );
        xmlfile.write ( "    <feature name=\"sis\"/>\n" );
        xmlfile.write ( "    <feature name=\"framecounter\"/>\n" );
        xmlfile.write ( "    <feature name=\"triggercounter\"/>\n" );
        xmlfile.write ( "    <feature name=\"maxisosize\"/>\n" );
        xmlfile.write ( "    <feature name=\"userprofiles\"/>\n" );
        xmlfile.write ( "  </featurecontrolpage>\n" );
        xmlfile.write ( "  <featurecontrolpage title=\"Autofnc\">\n" );
        xmlfile.write ( "    <feature name=\"autofncaoi\"/>\n" );
        xmlfile.write ( "    <feature name=\"autoshutter\"/>\n" );
        xmlfile.write ( "    <feature name=\"autogain\"/>\n" );
        xmlfile.write ( "  </featurecontrolpage>\n" );
        xmlfile.write ( "  <featurecontrolpage title=\"IO 1\">\n" );
        xmlfile.write ( "    <feature name=\"trigger\"/>\n" );
        xmlfile.write ( "    <feature name=\"triggerdelay\"/>\n" );
        xmlfile.write ( "    <feature name=\"io\"/>\n" );
        xmlfile.write ( "  </featurecontrolpage>\n" );
        xmlfile.write ( "  <featurecontrolpage title=\"IO 2\">\n" );
        xmlfile.write ( "    <feature name=\"inputdebounce\"/>\n" );
        xmlfile.write ( "    <feature name=\"pwmout\"/>\n" );
        xmlfile.write ( "  </featurecontrolpage>\n" );
        xmlfile.write ( "</featurecontrollayout>\n" );
        xmlfile.close();
    }
}

int main ( int argc, char *argv[] )
{
    checkProfile();

    Q_INIT_RESOURCE ( application );
    QApplication app ( argc, argv );

    qsrand(QTime::currentTime().msec());

    /* check permissions of /dev/raw1394 device node */
    QFileInfo deviceinfo ( "/dev/raw1394" );
    if ( deviceinfo.exists() )
    {
        if ( !deviceinfo.isReadable() || !deviceinfo.isWritable() )
        {
            QMessageBox msgBox;
            msgBox.setText("Insufficient permissions for device node /dev/raw1394 detected. Make sure that read/write-access is permitted.");
            msgBox.exec();
        }
    }

    MainWindow * mw = new MainWindow();

    mw->show();

    return app.exec();
}

