/** \file mainwindow.cpp
 *  \brief This file contains the implementation of the MainWindow class.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <QKeyEvent>
#include <QVector>
#include <QInputDialog>
#include <QVBoxLayout>
#include <QtXml>
#include <QMessageBox>
#include <QFile>
#include <QScrollArea>

#include "imageprocessingthread.h"
#include "mainwindow.h"
#include "smarttabbar.h"

#include "featurecontrols/standardfeaturecontrol.h"
#include "logwindow.h"
#include "camerainfodialog.h"
#include "registerwindow.h"
#include "cameralistfile.h"
#include "lockfile.h"

MainWindow::MainWindow( QWidget* parent, Qt::WFlags fl )
        : QMainWindow( parent, fl )
        , Ui::MainWindow()
        , m_pVideoSizeActions( NULL )
        , m_pVideoBayerPatternActions( NULL )
        , m_pVideoDebayerMethodActions( NULL )
        , m_pCamera( NULL )
{
    setupUi ( this );
    setWindowTitle( QString("cc1394 3.0") );

    m_pLayoutSplitter->setControlVisibility ( false );

    delete( m_pConfigTabWidget->widget(0) );

    loadFeatureLayout( QDir::home().absolutePath()+"/.cc1394/featurecontrollayout.xml" );

    m_pViewWidget->initVideo();

    /* connect a number of menu actions to separate slots */
    connect( ( SmartTabBar* ) m_pConfigTabWidget->tabBar(), SIGNAL( visibilityChanged ( bool ) ),
             m_pLayoutSplitter, SLOT( setControlVisibility ( bool ) ) );

    connect( m_pActionVideoKeepAR, SIGNAL( changed() ),
             this, SLOT( onVideoMenuArAction() ) );

    connect( m_pActionVideoFullScreen, SIGNAL( triggered() ),
             this, SLOT( onVideoMenuFullScreenAction() ) );

    /* image scaling control items */
    m_pVideoSizeActions = new QActionGroup ( this );

    m_pActionVideoFitToWindow->setData ( QVariant ( double ( -1 ) ) );
    m_pActionVideo25Percent->setData ( QVariant ( double ( 0.25 ) ) );
    m_pActionVideo50Percent->setData ( QVariant ( double ( 0.5 ) ) );
    m_pActionVideo100Percent->setData ( QVariant ( double ( 1 ) ) );
    m_pActionVideo200Percent->setData ( QVariant ( double ( 2 ) ) );
    m_pActionVideo400Percent->setData ( QVariant ( double ( 4 ) ) );
    m_pActionVideoCustomSize->setData ( QVariant ( double ( 0 ) ) );

    connect( m_pActionVideoFitToWindow , SIGNAL( triggered() ),
             this, SLOT( onVideoMenuScaleAction() ) );
    connect( m_pActionVideo25Percent, SIGNAL( triggered() ),
             this, SLOT( onVideoMenuScaleAction() ) );
    connect( m_pActionVideo50Percent, SIGNAL( triggered() ),
             this, SLOT( onVideoMenuScaleAction() ) );
    connect( m_pActionVideo100Percent, SIGNAL( triggered() ),
             this, SLOT( onVideoMenuScaleAction() ) );
    connect( m_pActionVideo200Percent, SIGNAL( triggered() ),
             this, SLOT( onVideoMenuScaleAction() ) );
    connect( m_pActionVideo400Percent, SIGNAL( triggered() ),
             this, SLOT( onVideoMenuScaleAction() ) );
    connect( m_pActionVideoCustomSize, SIGNAL( triggered() ),
             this, SLOT( onVideoMenuScaleAction() ) );

    m_pVideoSizeActions->addAction ( m_pActionVideoFitToWindow );
    m_pVideoSizeActions->addAction ( m_pActionVideo25Percent );
    m_pVideoSizeActions->addAction ( m_pActionVideo50Percent );
    m_pVideoSizeActions->addAction ( m_pActionVideo100Percent );
    m_pVideoSizeActions->addAction ( m_pActionVideo200Percent );
    m_pVideoSizeActions->addAction ( m_pActionVideo400Percent );
    m_pVideoSizeActions->addAction ( m_pActionVideoCustomSize );

    /* debayering control items */
    m_pVideoBayerPatternActions = new QActionGroup ( this );
    m_pVideoBayerPatternActions->addAction ( m_pActionBayerAuto );
    m_pVideoBayerPatternActions->addAction ( m_pActionBayerBGGR );
    m_pVideoBayerPatternActions->addAction ( m_pActionBayerGBRG );
    m_pVideoBayerPatternActions->addAction ( m_pActionBayerGRBG );
    m_pVideoBayerPatternActions->addAction ( m_pActionBayerRGGB );

    m_pActionBayerAuto->setData ( QVariant ( (int) CC1394_BAYERPATTERN_AUTO ) );
    m_pActionBayerBGGR->setData ( QVariant ( (int) CC1394_BAYERPATTERN_BGGR ) );
    m_pActionBayerGBRG->setData ( QVariant ( (int) CC1394_BAYERPATTERN_GBRG ) );
    m_pActionBayerGRBG->setData ( QVariant ( (int) CC1394_BAYERPATTERN_GRBG ) );
    m_pActionBayerRGGB->setData ( QVariant ( (int) CC1394_BAYERPATTERN_RGGB ) );

    connect( m_pActionBayerAuto, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onBayerPatternChange() ) );
    connect( m_pActionBayerBGGR, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onBayerPatternChange() ) );
    connect( m_pActionBayerGBRG, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onBayerPatternChange() ) );
    connect( m_pActionBayerGRBG, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onBayerPatternChange() ) );
    connect( m_pActionBayerRGGB, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onBayerPatternChange() ) );

    m_pVideoDebayerMethodActions = new QActionGroup( this );
    m_pVideoDebayerMethodActions->addAction( m_pActionMethodNearest );
    m_pVideoDebayerMethodActions->addAction( m_pActionMethodSimple );
    m_pVideoDebayerMethodActions->addAction( m_pActionMethodBilinear );
    m_pVideoDebayerMethodActions->addAction( m_pActionMethodHqLinear );
    m_pVideoDebayerMethodActions->addAction( m_pActionMethodDownsample );
    m_pVideoDebayerMethodActions->addAction( m_pActionMethodEdgeSense );
    m_pVideoDebayerMethodActions->addAction( m_pActionMethodVNG );
    m_pVideoDebayerMethodActions->addAction( m_pActionMethodAHD );
    m_pVideoDebayerMethodActions->addAction( m_pActionMethodAvtYuv422 );
    m_pVideoDebayerMethodActions->addAction( m_pActionMethodAvtLcaa );
    m_pVideoDebayerMethodActions->addAction( m_pActionMethodAvtLcaaV );

    m_pActionMethodNearest->setData ( QVariant ( (int) DC1394_BAYER_METHOD_NEAREST ) );
    m_pActionMethodSimple->setData ( QVariant ( (int) DC1394_BAYER_METHOD_SIMPLE ) );
    m_pActionMethodBilinear->setData ( QVariant ( (int) DC1394_BAYER_METHOD_BILINEAR ) );
    m_pActionMethodHqLinear->setData ( QVariant ( (int) DC1394_BAYER_METHOD_HQLINEAR ) );
    m_pActionMethodDownsample->setData ( QVariant ( (int) DC1394_BAYER_METHOD_DOWNSAMPLE ) );
    m_pActionMethodEdgeSense->setData ( QVariant ( (int) DC1394_BAYER_METHOD_EDGESENSE ) );
    m_pActionMethodVNG->setData ( QVariant ( (int) DC1394_BAYER_METHOD_VNG ) );
    m_pActionMethodAHD->setData ( QVariant ( (int) DC1394_BAYER_METHOD_AHD ) );
    m_pActionMethodAvtYuv422->setData ( QVariant ( (int) DC1394_BAYER_METHOD_AVT_YUV422 ) );
    m_pActionMethodAvtLcaa->setData ( QVariant ( (int) DC1394_BAYER_METHOD_AVT_LCAA ) );
    m_pActionMethodAvtLcaaV->setData ( QVariant ( (int) DC1394_BAYER_METHOD_AVT_LCAAV ) );

    connect( m_pActionMethodNearest, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onDebayerMethodChange() ) );
    connect( m_pActionMethodSimple, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onDebayerMethodChange() ) );
    connect( m_pActionMethodBilinear, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onDebayerMethodChange() ) );
    connect( m_pActionMethodHqLinear, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onDebayerMethodChange() ) );
    connect( m_pActionMethodDownsample, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onDebayerMethodChange() ) );
    connect( m_pActionMethodEdgeSense, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onDebayerMethodChange() ) );
    connect( m_pActionMethodVNG, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onDebayerMethodChange() ) );
    connect( m_pActionMethodAHD, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onDebayerMethodChange() ) );
    connect( m_pActionMethodAvtYuv422, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onDebayerMethodChange() ) );
    connect( m_pActionMethodAvtLcaa, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onDebayerMethodChange() ) );
    connect( m_pActionMethodAvtLcaaV, SIGNAL( triggered() ),
             m_pViewWidget, SLOT( onDebayerMethodChange() ) );

    connect( m_pActionDebayerEnabled, SIGNAL( triggered(bool) ),
             m_pViewWidget, SLOT( onDebayerEnableChange(bool) ) );

    connect( m_pActionInfoDebugOutput, SIGNAL( triggered() ),
             this, SLOT( onInfoMenuDebug() ) );

    connect( m_pActionInfoCamera, SIGNAL( triggered() ),
             this, SLOT( onInfoMenuCamInfo() ) );

    /* connect the whole camera menu to a single slot */
    connect( m_pMenuCameras, SIGNAL( triggered ( QAction* ) ),
             this, SLOT( onCameraMenuAction ( QAction* ) ) );

    connect( m_pViewWidget , SIGNAL( statisticsUpdate ( double, double, unsigned int, unsigned int ) ),
             this, SLOT( onStatsUpdate ( double, double, unsigned int, unsigned int ) ) );

    connect( m_pActionAbout, SIGNAL( triggered() ),
             this, SLOT( onHelpMenuAboutAction() ) );

    /* connect 'Control' menu actions */
    connect( m_pActionControlFreerun, SIGNAL(triggered()), this, SLOT(onControlMenuFreeRun()) );
    connect( m_pActionControlMultiShot, SIGNAL(triggered()), this, SLOT(onControlMenuMultiShot()) );
    connect( m_pActionControlSingleShot, SIGNAL(triggered()), this, SLOT(onControlMenuSingleShot()) );
    connect( m_pActionControlStop, SIGNAL(triggered()), this, SLOT(onControlMenuStop()) );
    connect( m_pActionControlRegisterAccess, SIGNAL(triggered()), this, SLOT(onControlMenuRegisterAccess()) );

    /* initialize statusbar */
    m_pStatusLabel = new QLabel;
    m_pStatusbar->insertPermanentWidget( 0, m_pStatusLabel, 1 );

    m_pAcquisitionThread = NULL;

    Dc1394Camera::init ();

    updateCameras();
    /* add random component to cameralist update interval - this should minimize the probability
       of multiple viewer instances interfering each other periodically */
    startTimer( 2000 + ((qrand()*300)/RAND_MAX) );
}

MainWindow::~MainWindow()
{
}


void MainWindow::updateCameras()
{
    /* get current camera list */
    dc1394error_t eRet;
    QVector<cc1394_camitem> camlist;

    LockFile lock(QDir::home().absolutePath()+"/.cc1394/lock", 3, 50, 8 );
    if( !lock.lock() )
    {
        return;
    }

    CameraListFile file;
    file.open( QDir::home().absolutePath()+"/.cc1394/cameralist" );

    QMap<uint64_t, camera_list_entry>* pmapCameras = file.getMapPtr();
    eRet = Dc1394Camera::updateCamList( *pmapCameras );
    if( DC1394_SUCCESS != eRet )
    {
        QMessageBox::critical( this, tr("Error"),
                               tr("Camera enumeration failed!") );
        file.close();
        lock.unlock();
        return;
    }

    /* remove locks we own and other locks when expired */
    QList<uint64_t> lstKeys = pmapCameras->keys();
    foreach( uint64_t id, lstKeys )
    {
         /* remove any camera locks from this process and remove outdated locks from other processes */
        if( (*pmapCameras)[id].bLocked && (*pmapCameras)[id].nLockingPid == QCoreApplication::applicationPid () )
        {
            (*pmapCameras)[id].bLocked = false;
        }
        /* 10 seconds timeout for locks issued by other processes */
        else if( (*pmapCameras)[id].bLocked && ( QDateTime::currentDateTime().toTime_t() - (*pmapCameras)[id].nLock_TimeStamp ) > 10 )
        {
            (*pmapCameras)[id].bLocked = false;
        }
        /* set lock for currently selected camera */
        if( NULL != m_pCamera && (*pmapCameras)[id].nGuid == m_pCamera->nGuid() )
        {
            (*pmapCameras)[id].bLocked = true;
            (*pmapCameras)[id].nLockingPid = QCoreApplication::applicationPid ();
            (*pmapCameras)[id].nLock_TimeStamp = QDateTime::currentDateTime().toTime_t();
        }
    }

    /* fill menu with cameras which are not listed yet */
    foreach ( camera_list_entry cam, *pmapCameras )
    {
        bool bExists = false;
        foreach( QAction *pExistingAction, m_lstCameraActions )
        {
            if( cam.nGuid == (uint64_t) pExistingAction->data().toULongLong() )
            {
                bExists = true;
                break;
            }
        }
        if( bExists )
        {
            continue;
        }
        QAction *action = m_pMenuCameras->addAction ( cam.strName );
        action->setData ( QVariant ( ( qulonglong ) cam.nGuid ) );
        action->setCheckable(true);
        m_lstCameraActions.append ( action );
    }

    /* delete all cameras that have been removed, disable cameras locked by other processes */
    int nCount = m_lstCameraActions.count();
    for( int i=nCount-1; i>=0; i-- )
    {
        QAction* pExistingAction = m_lstCameraActions[i];
        if( !pmapCameras->contains( (uint64_t) pExistingAction->data().toULongLong() ) )
        {
            /* removed camera still in use? */
            if( NULL != m_pCamera &&
                (uint64_t) pExistingAction->data().toULongLong() == m_pCamera->nGuid() )
            {
                /* try to close camera */
                if ( m_pAcquisitionThread != NULL )
                {
                    m_pAcquisitionThread->abort(true);
                    delete m_pAcquisitionThread;
                    m_pAcquisitionThread = NULL;
                }

                setWindowTitle( QString("cc1394 3.0") );

                foreach( ControlContainer* pPage, m_lstControlContainers )
                {
                    pPage->setCamera( NULL, NULL );
                }

                delete m_pCamera;
                m_pCamera = NULL;

                m_pActionInfoCamera->setEnabled( false );
                m_pActionControlFreerun->setEnabled( false );
                m_pActionControlMultiShot->setEnabled( false );
                m_pActionControlSingleShot->setEnabled( false );
                m_pActionControlStop->setEnabled( false );
                m_pActionControlRegisterAccess->setEnabled( false );
            }
            delete( pExistingAction );
            m_lstCameraActions.removeOne( pExistingAction );
        }
        else
        {
            pExistingAction->setEnabled( !(*pmapCameras)[(uint64_t) pExistingAction->data().toULongLong()].bLocked ||
                                         (*pmapCameras)[(uint64_t) pExistingAction->data().toULongLong()].nLockingPid == QCoreApplication::applicationPid () );
        }
    }
    file.close();
    lock.unlock();
}


void MainWindow::resizeEvent ( QResizeEvent* /*event*/ )
{
    m_pViewWidget->resizeOverlay();
}


void MainWindow::keyPressEvent ( QKeyEvent * event )
{
    /* disable fullscreen, does not work as intended yet */
    /*
    if ( event->key() == Qt::Key_F )
    {
        m_pViewWidget->setFullScreen ( !m_pViewWidget->fullScreen() );
    }
    */
}

void MainWindow::timerEvent(QTimerEvent* /*event*/)
{
    updateCameras();
}


bool MainWindow::loadFeatureLayout ( QString strFilename )
{
    /* open xml file */
    QDomDocument doc ( "featurelayout" );
    QFile file ( strFilename );
    if ( !file.open ( QIODevice::ReadOnly ) )
        {
            QMessageBox::critical( this, tr("Error"),
                                   QString(tr("Could not open file: ")) + strFilename );
            return false;
        }
    if ( !doc.setContent ( &file ) )
    {
        QMessageBox::critical( this, tr("Error"),
                               QString(tr("Could not parse file: ")) + strFilename );
        return false;
    }
    file.close();

    /* parse dom tree and fill controls */
    QDomElement docElem = doc.documentElement();

    QDomElement pageNode = docElem.firstChildElement("featurecontrolpage");
    while ( !pageNode.isNull() )
    {
        /* create new control tab page */
        QScrollArea* pScrollArea = new QScrollArea( m_pConfigTabWidget );
        pScrollArea->setSizePolicy( QSizePolicy( QSizePolicy::Ignored, QSizePolicy::Ignored ) );
        pScrollArea->setFrameShape( QFrame::NoFrame );

        ControlContainer* pNewTabPage = new ControlContainer( NULL );
        pScrollArea->setWidget( pNewTabPage );
        pNewTabPage->setAutoFillBackground( false );
        pScrollArea->setWidgetResizable( true );
        m_lstControlContainers.append( pNewTabPage );
        m_pConfigTabWidget->addTab( pScrollArea, pageNode.attribute("title","untitled") );

        /* fill new page with controls */
        QDomElement controlNode = pageNode.firstChildElement("feature");
        while ( !controlNode.isNull() )
        {
            bool bRet = pNewTabPage->addControl( controlNode.attribute("name") );
            if ( !bRet )
            {
                QMessageBox::critical( this, tr("Error"),
                                       QString(tr("Could not create control: ")) + 
                                       controlNode.attribute("name") );
            }
            controlNode = controlNode.nextSiblingElement("feature");
        }
        pNewTabPage->addSpacerItem();
        pageNode = pageNode.nextSiblingElement("featurecontrolpage");
    }
    return true;
}


void MainWindow::onCameraMenuAction ( QAction* pAction )
{
    uint64_t nCamGuid = ( uint64_t ) pAction->data().toLongLong();

    if ( m_pCamera != NULL && nCamGuid == m_pCamera->nGuid() )
    {
        /* invalid entry or currently opened camera has been selected, do nothing */
        return;
    }

    /* free last camera's resources  */
    if( NULL!=m_pAcquisitionThread )
    {
        m_pAcquisitionThread->abort(true);
        delete m_pAcquisitionThread;
        m_pAcquisitionThread = NULL;
    }

    if( NULL != m_pCamera )
    {
        delete m_pCamera;
        m_pCamera = NULL;
    }

    /* clear old checkmarks, check selected camera entry */
    foreach( QAction *pExistingAction, m_lstCameraActions )
    {
        pExistingAction->setChecked( nCamGuid == (uint64_t) pExistingAction->data().toULongLong() );
    }

    /*  open camera   */
    try
    {
        m_pCamera = new Dc1394Camera ( nCamGuid );
    }
    catch ( QString strErr )
    {
        m_pCamera = NULL;
        QMessageBox::critical ( this, "Error", QString ( "camera object creation failed: " ) + strErr );

        foreach( ControlContainer* pPage, m_lstControlContainers )
        {
            pPage->setCamera( NULL, NULL );
        }

        setWindowTitle( "cc1394 3.0" );

        return;
    }

    m_pAcquisitionThread = new AcquisitionThread ( this, m_pCamera );

    /* direct signal-signal connection!
       See definition of SdlVideoWidget::newImageReceived for details. */
    connect ( m_pAcquisitionThread,
              SIGNAL ( newImageReceived(dc1394video_frame_t*,QSemaphore*,QSemaphore*)),
              m_pViewWidget,
              SIGNAL ( newImageReceived(dc1394video_frame_t*,QSemaphore*,QSemaphore*)),
              Qt::DirectConnection );

    /* update window title and statusbar */
    setWindowTitle( QString("cc1394 3.0 - ") + pAction->text() );
    m_pStatusLabel->setText( tr ( "Frames received: 0, (fps: 0)" ) );

    m_pCamera->pollStateFromCamera();

    /* announce new camera selection to feature controls */
    foreach( ControlContainer* pPage, m_lstControlContainers )
    {
        pPage->setCamera( m_pCamera, m_pAcquisitionThread );
    }

    connect ( m_pAcquisitionThread, SIGNAL ( acquisitionStateChanged ( bool ) ),
              this, SLOT ( onAcquisitionStateChange ( bool ) ) );

    m_pActionInfoCamera->setEnabled( true );
    m_pActionControlFreerun->setEnabled( true );
    m_pActionControlMultiShot->setEnabled( true );
    m_pActionControlSingleShot->setEnabled( true );
    m_pActionControlStop->setEnabled( false );
    m_pActionControlRegisterAccess->setEnabled( true );
}


void MainWindow::onVideoMenuScaleAction()
{
    if ( ( ( QAction* ) sender() )->data().toDouble() != 0 )
    {
        m_pViewWidget->setScale ( ( ( QAction* ) sender() )->data().toDouble() );
    }
    else
    {
        bool bOk;
        double dInput = QInputDialog::getDouble ( this,
                        tr ( "Custom Zoom Factor" ),
                        tr ( "Zoom Factor:" ),
                        ( -1==m_pViewWidget->scale() ) ? 1 : m_pViewWidget->scale(),
                        0.001, 100, 3, &bOk );
        if ( bOk )
        {
            m_pViewWidget->setScale ( dInput );
        }
        switch ( ( int ) ( m_pViewWidget->scale() *100 ) )
        {
            case -100:
                m_pActionVideoFitToWindow->setChecked ( true );
                break;
            case 25:
                m_pActionVideo25Percent->setChecked ( true );
                break;
            case 50:
                m_pActionVideo50Percent->setChecked ( true );
                break;
            case 100:
                m_pActionVideo100Percent->setChecked ( true );
                break;
            case 200:
                m_pActionVideo200Percent->setChecked ( true );
                break;
            case 400:
                m_pActionVideo400Percent->setChecked ( true );
                break;
            default:
                m_pActionVideoCustomSize->setChecked ( true );
        }
    }
}


void MainWindow::onVideoMenuArAction()
{
    m_pViewWidget->setAspectRatioFixed ( ( ( QAction* ) sender() ) -> isChecked() );
}


void MainWindow::onVideoMenuFullScreenAction()
{
    m_pViewWidget->setFullScreen ( !m_pViewWidget->fullScreen() );
}

void MainWindow::onControlMenuFreeRun()
{
    if( m_pAcquisitionThread==NULL || m_pCamera==NULL )
    {
        return;
    }
    if( !m_pCamera->FormatValid() )
    {
        QMessageBox::critical ( this, "Error", QString ( "Configuration error: " ) + m_pCamera->GetFormatErrorDesc() );
        return;
    }

    m_pAcquisitionThread->setAcquisitionMode( CC1394_ACQ_FREERUN );
    m_pAcquisitionThread->start();
    /* no idea why this repaint is needed, but it is... */
    m_pConfigTabWidget->repaint();
}

void MainWindow::onControlMenuSingleShot()
{
    if( m_pAcquisitionThread==NULL || m_pCamera==NULL )
    {
        return;
    }
    if( !m_pCamera->FormatValid() )
    {
        QMessageBox::critical ( this, "Error", QString ( "Configuration error: " ) + m_pCamera->GetFormatErrorDesc() );
        return;
    }

    m_pAcquisitionThread->setAcquisitionMode( CC1394_ACQ_SINGLE_SHOT );
    m_pAcquisitionThread->start();
    /* no idea why this repaint is needed, but it is... */
    m_pConfigTabWidget->repaint();
}

void MainWindow::onControlMenuMultiShot()
{
    if( m_pAcquisitionThread==NULL || m_pCamera==NULL )
    {
        return;
    }
    if( !m_pCamera->FormatValid() )
    {
        QMessageBox::critical ( this, "Error", QString ( "Configuration error: " ) + m_pCamera->GetFormatErrorDesc() );
        return;
    }

    m_pAcquisitionThread->setAcquisitionMode( CC1394_ACQ_MULTI_SHOT );
    m_pAcquisitionThread->start();
    /* no idea why this repaint is needed, but it is... */
    m_pConfigTabWidget->repaint();
}

void MainWindow::onControlMenuStop()
{
    m_pAcquisitionThread->abort();
    /* no idea why this repaint is needed, but it is... */
    m_pConfigTabWidget->repaint();
}

void MainWindow::onControlMenuRegisterAccess()
{
    RegisterWindow* pWin = new RegisterWindow( this, m_pCamera );
    pWin->show();
}


void MainWindow::onInfoMenuDebug()
{
    LogWindow* pLogWin = new LogWindow( this );
    Logger::connectLogWindow( pLogWin );
    pLogWin->show();
}


void MainWindow::onInfoMenuCamInfo()
{
    CameraInfoDialog* pCamInfoDialog = new CameraInfoDialog( this, m_pCamera );
    pCamInfoDialog->exec();
    delete pCamInfoDialog;
}

void MainWindow::onStatsUpdate ( double dbFramesPerSecond,
                                 double dbFramesPerSecond_Displayed,
                                 unsigned int nFrameCount ,
                                 unsigned int nFrameCount_Displayed )
{
    m_pStatusLabel->setText( QString( tr("Frames received: %1, displayed: %2, fps: %3, fps(disp): %4") )
                             .arg( nFrameCount, 6 )
                             .arg( nFrameCount_Displayed, 6 )
                             .arg( dbFramesPerSecond, 6, 'f', 2 )
                             .arg( dbFramesPerSecond_Displayed, 6, 'f', 2 ) );
}


void MainWindow::onHelpMenuAboutAction()
{
    QMessageBox::about ( this, "About Dialog", "cc1394 Viewer Version 3.0\n(c) Allied Vision Technologies 2010" );
}


void MainWindow::onAcquisitionStateChange ( bool bAcquisitionActive )
{
    m_pViewWidget->setAcquisitionState ( bAcquisitionActive );

    m_pActionControlFreerun->setEnabled( !bAcquisitionActive );
    m_pActionControlMultiShot->setEnabled( !bAcquisitionActive );
    m_pActionControlSingleShot->setEnabled( !bAcquisitionActive );
    m_pActionControlStop->setEnabled( bAcquisitionActive );

    foreach( ControlContainer* pPage, m_lstControlContainers )
    {
        pPage->setAcquisitionState( bAcquisitionActive );
    }
}
